@@ -40,6 +40,7 @@
 
 #include "command_executor.hh"
 #include "db_sub_source.hh"
+#include "papertrail_proc.hh"
 
 using namespace std;
 
@@ -270,20 +271,21 @@ string execute_sql(exec_context &ec, const string &sql, string &alt_msg)
             }
         }
 
-        if (lnav_data.ld_rl_view != NULL) {
+        if (lnav_data.ld_rl_view != nullptr) {
             lnav_data.ld_rl_view->set_value("");
         }
     }
 
     gettimeofday(&end_tv, NULL);
     if (retcode == SQLITE_DONE) {
+        lnav_data.ld_filter_view.reload_data();
         lnav_data.ld_views[LNV_DB].reload_data();
         lnav_data.ld_views[LNV_DB].set_left(0);
 
         if (!ec.ec_accumulator.empty()) {
             retval = ec.ec_accumulator.get_string();
         }
-        else if (dls.dls_rows.size() > 0) {
+        else if (!dls.dls_rows.empty()) {
             vis_bookmarks &bm = lnav_data.ld_views[LNV_LOG].get_bookmarks();
 
             if (lnav_data.ld_flags & LNF_HEADLESS) {
@@ -354,7 +356,7 @@ string execute_sql(exec_context &ec, const string &sql, string &alt_msg)
     if (!(lnav_data.ld_flags & LNF_HEADLESS)) {
         lnav_data.ld_bottom_source.update_loading(0, 0);
         lnav_data.ld_status[LNS_BOTTOM].do_update();
-        redo_search(LNV_DB);
+        lnav_data.ld_views[LNV_DB].redo_search();
     }
 
     return retval;
@@ -384,6 +386,7 @@ static string execute_file_contents(exec_context &ec, const string &path, bool m
     pair<string, string> dir_and_base = split_path(path);
 
     ec.ec_path_stack.push_back(dir_and_base.first);
+    ec.ec_output_stack.emplace_back(nonstd::nullopt);
     while ((line_size = getline(&line, &line_max_size, file)) != -1) {
         line_number += 1;
 
@@ -430,6 +433,7 @@ static string execute_file_contents(exec_context &ec, const string &path, bool m
     } else {
         fclose(file);
     }
+    ec.ec_output_stack.pop_back();
     ec.ec_path_stack.pop_back();
 
     return retval;
@@ -512,10 +516,8 @@ string execute_file(exec_context &ec, const string &path_and_args, bool multilin
         }
 
         if (!paths_to_exec.empty()) {
-            for (vector<script_metadata>::iterator path_iter = paths_to_exec.begin();
-                 path_iter != paths_to_exec.end();
-                 ++path_iter) {
-                result = execute_file_contents(ec, path_iter->sm_path, multiline);
+            for (auto &path_iter : paths_to_exec) {
+                result = execute_file_contents(ec, path_iter.sm_path, multiline);
             }
             retval = result;
         }
@@ -539,12 +541,9 @@ string execute_from_file(exec_context &ec, const string &path, int line_number,
             retval = execute_command(ec, cmdline);
             break;
         case '/':
-            if (!lnav_data.ld_view_stack.empty()) {
-                lnav_view_t index = lnav_view_t(lnav_data.ld_view_stack.back() - lnav_data.ld_views);
-
-                execute_search(index, cmdline.substr(1));
-                retval = "";
-            }
+            lnav_data.ld_view_stack.top() | [cmdline] (auto tc) {
+                tc->execute_search(cmdline.substr(1));
+            };
             break;
         case ';':
             setup_logline_table(ec);
@@ -558,9 +557,8 @@ string execute_from_file(exec_context &ec, const string &path, int line_number,
             break;
     }
 
-    if (rescan_files()) {
-        rebuild_indexes(true);
-    }
+    rescan_files();
+    rebuild_indexes();
 
     log_info("%s:%d:execute result -- %s",
             path.c_str(),
@@ -581,12 +579,9 @@ string execute_any(exec_context &ec, const string &cmdline_with_mode)
             retval = execute_command(ec, cmdline);
             break;
         case '/':
-            if (!lnav_data.ld_view_stack.empty()) {
-                lnav_view_t index = lnav_view_t(lnav_data.ld_view_stack.back() - lnav_data.ld_views);
-
-                execute_search(index, cmdline.substr(1));
-                retval = "";
-            }
+            lnav_data.ld_view_stack.top() | [cmdline] (auto tc) {
+                tc->execute_search(cmdline.substr(1));
+            };
             break;
         case ';':
             setup_logline_table(ec);
@@ -601,9 +596,8 @@ string execute_any(exec_context &ec, const string &cmdline_with_mode)
             break;
     }
 
-    if (rescan_files()) {
-        rebuild_indexes(true);
-    }
+    rescan_files();
+    rebuild_indexes();
 
     return retval;
 }
@@ -629,11 +623,9 @@ void execute_init_commands(exec_context &ec, vector<pair<string, string> > &msgs
             msg = execute_command(ec, cmd.substr(1));
             break;
         case '/':
-            if (!lnav_data.ld_view_stack.empty()) {
-                lnav_view_t index = lnav_view_t(lnav_data.ld_view_stack.back() - lnav_data.ld_views);
-
-                execute_search(index, cmd.substr(1));
-            }
+            lnav_data.ld_view_stack.top() | [cmd] (auto tc) {
+                tc->execute_search(cmd.substr(1));
+            };
             break;
         case ';':
             setup_logline_table(ec);
@@ -644,11 +636,10 @@ void execute_init_commands(exec_context &ec, vector<pair<string, string> > &msgs
             break;
         }
 
-        msgs.push_back(make_pair(msg, alt_msg));
+        msgs.emplace_back(msg, alt_msg);
 
-        if (rescan_files()) {
-            rebuild_indexes(true);
-        }
+        rescan_files();
+        rebuild_indexes();
 
         ec.ec_source.pop();
     }
@@ -712,10 +703,22 @@ int sql_callback(exec_context &ec, sqlite3_stmt *stmt)
         }
     }
     for (lpc = 0; lpc < ncols; lpc++) {
-        const char *value     = (const char *)sqlite3_column_text(stmt, lpc);
+        const char *value = (const char *)sqlite3_column_text(stmt, lpc);
+        db_label_source::header_meta &hm = dls.dls_headers[lpc];
 
         dls.push_column(value);
-        if (value != NULL &&
+        if ((hm.hm_column_type == SQLITE_TEXT ||
+             hm.hm_column_type == SQLITE_NULL) && hm.hm_sub_type == 0) {
+            sqlite3_value *raw_value = sqlite3_column_value(stmt, lpc);
+
+            switch (sqlite3_value_type(raw_value)) {
+                case SQLITE_TEXT:
+                    hm.hm_column_type = SQLITE_TEXT;
+                    hm.hm_sub_type = sqlite3_value_subtype(raw_value);
+                    break;
+            }
+        }
+        if (value != nullptr &&
             (dls.dls_headers[lpc].hm_name == "log_line" ||
              strstr(dls.dls_headers[lpc].hm_name.c_str(), "log_line"))) {
             int line_number = -1;
@@ -732,29 +735,50 @@ int sql_callback(exec_context &ec, sqlite3_stmt *stmt)
 
 future<string> pipe_callback(exec_context &ec, const string &cmdline, auto_fd &fd)
 {
-    auto pp = make_shared<piper_proc>(fd, false);
-    static int exec_count = 0;
-    char desc[128];
+    auto out = ec.get_output();
 
-    lnav_data.ld_pipers.push_back(pp);
-    snprintf(desc,
-             sizeof(desc), "[%d] Output of %s",
-             exec_count++,
-             cmdline.c_str());
-    lnav_data.ld_file_names[desc]
-        .with_fd(pp->get_fd())
-        .with_detect_format(false);
-    lnav_data.ld_files_to_front.emplace_back(desc, 0);
-    if (lnav_data.ld_rl_view != NULL) {
-        lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(
-                                                X, "to close the file"));
+    if (out) {
+        FILE *file = *out;
+
+        return std::async(std::launch::async, [&fd, file]() {
+            char buffer[1024];
+            ssize_t rc;
+
+            if (file == stdout) {
+                lnav_data.ld_stdout_used = true;
+            }
+
+            while ((rc = read(fd, buffer, sizeof(buffer))) > 0) {
+                fwrite(buffer, rc, 1, file);
+            }
+
+            return string();
+        });
+    } else {
+        auto pp = make_shared<piper_proc>(fd, false);
+        static int exec_count = 0;
+        char desc[128];
+
+        lnav_data.ld_pipers.push_back(pp);
+        snprintf(desc,
+                 sizeof(desc), "[%d] Output of %s",
+                 exec_count++,
+                 cmdline.c_str());
+        lnav_data.ld_file_names[desc]
+            .with_fd(pp->get_fd())
+            .with_detect_format(false);
+        lnav_data.ld_files_to_front.emplace_back(desc, 0);
+        if (lnav_data.ld_rl_view != nullptr) {
+            lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(
+                                                    X, "to close the file"));
+        }
+
+        packaged_task<string()> task([]() { return ""; });
+
+        task();
+
+        return task.get_future();
     }
-
-    packaged_task<string()> task([]() { return ""; });
-
-    task();
-
-    return task.get_future();
 }
 
 void add_global_vars(exec_context &ec)
