@@ -40,15 +40,15 @@
 #include "session_data.hh"
 #include "command_executor.hh"
 #include "termios_guard.hh"
+#include "readline_callbacks.hh"
 #include "readline_possibilities.hh"
+#include "readline_highlighters.hh"
 #include "field_overlay_source.hh"
 #include "hotkeys.hh"
 #include "log_format_loader.hh"
 
 using namespace std;
 
-static bookmark_type_t BM_EXAMPLE("");
-
 class logline_helper {
 
 public:
@@ -59,7 +59,7 @@ public:
     logline &move_to_msg_start() {
         content_line_t cl = this->lh_sub_source.at(this->lh_current_line);
         std::shared_ptr<logfile> lf = this->lh_sub_source.find(cl);
-        logfile::iterator ll = lf->begin() + cl;
+        auto ll = lf->begin() + cl;
         while (ll->is_continued()) {
             --ll;
             --this->lh_current_line;
@@ -80,12 +80,14 @@ public:
         this->lh_line_values.clear();
         content_line_t cl = this->lh_sub_source.at(this->lh_current_line);
         std::shared_ptr<logfile> lf = this->lh_sub_source.find(cl);
-        logfile::iterator ll = lf->begin() + cl;
+        auto ll = lf->begin() + cl;
         log_format *format = lf->get_format();
         lf->read_full_message(ll, this->lh_msg_buffer);
-        format->annotate(this->lh_msg_buffer,
+        format->annotate(cl,
+                         this->lh_msg_buffer,
                          this->lh_string_attrs,
-                         this->lh_line_values);
+                         this->lh_line_values,
+                         false);
     };
 
     std::string to_string(const struct line_range &lr) {
@@ -101,60 +103,88 @@ public:
     vector<logline_value> lh_line_values;
 };
 
-/* XXX For one, this code is kinda crappy.  For two, we should probably link
- * directly with X so we don't need to have xclip installed and it'll work if
- * we're ssh'd into a box.
- */
-static void copy_to_xclip(void)
+static int key_sql_callback(exec_context &ec, sqlite3_stmt *stmt)
 {
-    textview_curses *tc = lnav_data.ld_view_stack.back();
-
-    bookmark_vector<vis_line_t> &bv =
-            tc->get_bookmarks()[&textview_curses::BM_USER];
-    bookmark_vector<vis_line_t>::iterator iter;
-    auto_mem<FILE> pfile(pclose);
-    int    line_count = 0;
-    string line;
-
-    pfile = open_clipboard(CT_GENERAL);
-
-    if (!pfile.in()) {
-        alerter::singleton().chime();
-        lnav_data.ld_rl_view->set_value(
-                "error: Unable to copy to clipboard.  "
-                        "Make sure xclip or pbcopy is installed.");
-        return;
+    if (!sqlite3_stmt_busy(stmt)) {
+        return 0;
     }
 
-    for (iter = bv.begin(); iter != bv.end(); iter++) {
-        tc->grep_value_for_line(*iter, line);
-        fprintf(pfile, "%s\n", line.c_str());
-        line_count += 1;
+    int ncols = sqlite3_column_count(stmt);
+
+    auto &vars = ec.ec_local_vars.top();
+
+    for (int lpc = 0; lpc < ncols; lpc++) {
+        const char *column_name = sqlite3_column_name(stmt, lpc);
+
+        if (sql_ident_needs_quote(column_name)) {
+            continue;
+        }
+        if (sqlite3_column_type(stmt, lpc) == SQLITE_NULL) {
+            continue;
+        }
+
+        vars[column_name] = string((const char *) sqlite3_column_text(stmt, lpc));
     }
 
-    char buffer[128];
+    return 0;
+}
 
-    snprintf(buffer, sizeof(buffer),
-             "Copied " ANSI_BOLD("%d") " lines to the clipboard",
-             line_count);
-    lnav_data.ld_rl_view->set_value(buffer);
+bool handle_keyseq(const char *keyseq)
+{
+    key_map &km = lnav_config.lc_ui_keymaps[lnav_config.lc_ui_keymap];
+
+    const auto &iter = km.km_seq_to_cmd.find(keyseq);
+    if (iter != km.km_seq_to_cmd.end()) {
+        vector<logline_value> values;
+        exec_context ec(&values, key_sql_callback, pipe_callback);
+        auto &var_stack = ec.ec_local_vars;
+        string result;
+
+        ec.ec_global_vars = lnav_data.ld_exec_context.ec_global_vars;
+        var_stack.push(map<string, string>());
+        auto &vars = var_stack.top();
+        vars["keyseq"] = keyseq;
+        for (const string &cmd : iter->second) {
+            log_debug("executing key sequence x%02x: %s",
+                      keyseq, cmd.c_str());
+            result = execute_any(ec, cmd);
+        }
+
+        lnav_data.ld_rl_view->set_value(result);
+        return true;
+    }
+
+    return false;
 }
 
 void handle_paging_key(int ch)
 {
-    if (lnav_data.ld_view_stack.empty()) {
+    if (lnav_data.ld_view_stack.vs_views.empty()) {
         return;
     }
 
-    textview_curses *   tc  = lnav_data.ld_view_stack.back();
+    textview_curses *tc = *lnav_data.ld_view_stack.top();
     exec_context &ec = lnav_data.ld_exec_context;
     logfile_sub_source *lss = NULL;
+    text_sub_source *tc_tss = tc->get_sub_source();
     bookmarks<vis_line_t>::type &     bm  = tc->get_bookmarks();
 
     if (tc->handle_key(ch)) {
         return;
     }
 
+    char keyseq[16];
+
+    snprintf(keyseq, sizeof(keyseq), "x%02x", ch);
+
+    if (handle_keyseq(keyseq)) {
+        return;
+    }
+
+    if (tc->handle_key(ch)) {
+        return;
+    }
+
     lss = dynamic_cast<logfile_sub_source *>(tc->get_sub_source());
 
     /* process the command keystroke */
@@ -181,13 +211,13 @@ void handle_paging_key(int ch)
             }
             else {
                 textview_curses *tc = lnav_data.ld_last_view;
-                textview_curses *top_tc = lnav_data.ld_view_stack.back();
-                text_time_translator *dst_view = dynamic_cast<text_time_translator *>(tc->get_sub_source());
-                text_time_translator *src_view = dynamic_cast<text_time_translator *>(top_tc->get_sub_source());
+                textview_curses *top_tc = *lnav_data.ld_view_stack.top();
+                auto *dst_view = dynamic_cast<text_time_translator *>(tc->get_sub_source());
+                auto *src_view = dynamic_cast<text_time_translator *>(top_tc->get_sub_source());
 
                 lnav_data.ld_last_view = NULL;
                 if (src_view != NULL && dst_view != NULL) {
-                    time_t top_time = src_view->time_for_row(top_tc->get_top());
+                    struct timeval top_time = src_view->time_for_row(top_tc->get_top());
 
                     tc->set_top(vis_line_t(dst_view->row_for_time(top_time)));
                 }
@@ -210,12 +240,6 @@ void handle_paging_key(int ch)
             }
             break;
 
-        case 'c':
-            copy_to_xclip();
-            lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(
-                    C, "to clear marked messages"));
-            break;
-
         case 'C':
             if (lss) {
                 lss->text_clear_marks(&textview_curses::BM_USER);
@@ -281,7 +305,6 @@ void handle_paging_key(int ch)
 
                 if (!tss.empty()) {
                     tss.rotate_left();
-                    redo_search(LNV_TEXT);
                 }
             }
             break;
@@ -295,7 +318,6 @@ void handle_paging_key(int ch)
 
                 if (!tss.empty()) {
                     tss.rotate_right();
-                    redo_search(LNV_TEXT);
                 }
             }
             break;
@@ -368,7 +390,7 @@ void handle_paging_key(int ch)
         case 'J':
             if (lnav_data.ld_last_user_mark.find(tc) ==
                 lnav_data.ld_last_user_mark.end() ||
-                !tc->is_visible(vis_line_t(lnav_data.ld_last_user_mark[tc]))) {
+                !tc->is_line_visible(vis_line_t(lnav_data.ld_last_user_mark[tc]))) {
                 lnav_data.ld_select_start[tc] = tc->get_top();
                 lnav_data.ld_last_user_mark[tc] = tc->get_top();
             }
@@ -402,7 +424,7 @@ void handle_paging_key(int ch)
 
             if (lnav_data.ld_last_user_mark.find(tc) ==
                 lnav_data.ld_last_user_mark.end() ||
-                !tc->is_visible(vis_line_t(lnav_data.ld_last_user_mark[tc]))) {
+                !tc->is_line_visible(vis_line_t(lnav_data.ld_last_user_mark[tc]))) {
                 new_mark = tc->get_top();
             }
             else {
@@ -531,10 +553,10 @@ void handle_paging_key(int ch)
 
         case '0':
             if (lss) {
-                time_t     first_time = lnav_data.ld_top_time;
+                struct timeval first_time = lss->time_for_row(tc->get_top());
                 int        step       = 24 * 60 * 60;
                 vis_line_t line       =
-                        lss->find_from_time(roundup_size(first_time, step));
+                        lss->find_from_time(roundup_size(first_time.tv_sec, step));
 
                 tc->set_top(line);
             }
@@ -542,7 +564,8 @@ void handle_paging_key(int ch)
 
         case ')':
             if (lss) {
-                time_t     day  = rounddown(lnav_data.ld_top_time, 24 * 60 * 60);
+                struct timeval first_time = lss->time_for_row(tc->get_top());
+                time_t     day  = rounddown(first_time.tv_sec, 24 * 60 * 60);
                 vis_line_t line = lss->find_from_time(day);
 
                 --line;
@@ -555,8 +578,9 @@ void handle_paging_key(int ch)
                 alerter::singleton().chime();
             }
             else if (lss) {
+                struct timeval first_time = lss->time_for_row(tc->get_top());
                 int        step     = ch == 'D' ? (24 * 60 * 60) : (60 * 60);
-                time_t     top_time = lnav_data.ld_top_time;
+                time_t     top_time = first_time.tv_sec;
                 vis_line_t line     = lss->find_from_time(top_time - step);
 
                 if (line != 0) {
@@ -570,9 +594,10 @@ void handle_paging_key(int ch)
 
         case 'd':
             if (lss) {
+                struct timeval first_time = lss->time_for_row(tc->get_top());
                 int        step = ch == 'd' ? (24 * 60 * 60) : (60 * 60);
                 vis_line_t line =
-                        lss->find_from_time(lnav_data.ld_top_time + step);
+                        lss->find_from_time(first_time.tv_sec + step);
 
                 tc->set_top(line);
 
@@ -660,7 +685,7 @@ void handle_paging_key(int ch)
                 textview_curses &   log_view = lnav_data.ld_views[LNV_LOG];
                 content_line_t      cl       = lss.at(log_view.get_top());
                 std::shared_ptr<logfile>           lf       = lss.find(cl);
-                logfile::iterator ll = lf->begin() + cl;
+                auto ll = lf->begin() + cl;
                 log_data_helper ldh(lss);
 
                 lnav_data.ld_exec_context.ec_top_line = tc->get_top();
@@ -673,24 +698,20 @@ void handle_paging_key(int ch)
                 if (tc == &lnav_data.ld_views[LNV_DB]) {
                     db_label_source &dls = lnav_data.ld_db_row_source;
 
-                    for (vector<db_label_source::header_meta>::iterator iter = dls.dls_headers.begin();
-                         iter != dls.dls_headers.end();
-                         ++iter) {
-                        if (!iter->hm_graphable) {
+                    for (auto &dls_header : dls.dls_headers) {
+                        if (!dls_header.hm_graphable) {
                             continue;
                         }
 
                         lnav_data.ld_rl_view->add_possibility(LNM_COMMAND,
                                                               "numeric-colname",
-                                                              iter->hm_name);
+                                                              dls_header.hm_name);
                     }
                 }
                 else {
-                    for (vector<logline_value>::iterator iter = ldh.ldh_line_values.begin();
-                         iter != ldh.ldh_line_values.end();
-                         ++iter) {
-                        const logline_value_stats *stats = iter->lv_format->stats_for_value(
-                            iter->lv_name);
+                    for (auto &ldh_line_value : ldh.ldh_line_values) {
+                        const logline_value_stats *stats = ldh_line_value.lv_format->stats_for_value(
+                            ldh_line_value.lv_name);
 
                         if (stats == NULL) {
                             continue;
@@ -698,14 +719,13 @@ void handle_paging_key(int ch)
 
                         lnav_data.ld_rl_view->add_possibility(LNM_COMMAND,
                                                               "numeric-colname",
-                                                              iter->lv_name.to_string());
+                                                              ldh_line_value.lv_name.to_string());
                     }
                 }
 
-                for (vector<string>::iterator iter = ldh.ldh_namer->cn_names.begin();
-                     iter != ldh.ldh_namer->cn_names.end();
-                     ++iter) {
-                    lnav_data.ld_rl_view->add_possibility(LNM_COMMAND, "colname", *iter);
+                for (auto &cn_name : ldh.ldh_namer->cn_names) {
+                    lnav_data.ld_rl_view->add_possibility(LNM_COMMAND, "colname",
+                                                          cn_name);
                 }
                 for (auto iter : ldh.ldh_namer->cn_builtin_names) {
                     if (iter == "col") {
@@ -746,10 +766,9 @@ void handle_paging_key(int ch)
                 }
             }
 
-            add_view_text_possibilities(LNM_COMMAND, "filter", tc);
-            lnav_data.ld_rl_view->
-                    add_possibility(LNM_COMMAND, "filter",
-                                    lnav_data.ld_last_search[tc - lnav_data.ld_views]);
+            lnav_data.ld_doc_status_source.set_title("Command Help");
+            add_view_text_possibilities(lnav_data.ld_rl_view, LNM_COMMAND, "filter", tc);
+            lnav_data.ld_rl_view->add_possibility(LNM_COMMAND, "filter", tc->get_last_search());
             add_filter_possibilities(tc);
             add_mark_possibilities();
             add_config_possibilities();
@@ -761,10 +780,12 @@ void handle_paging_key(int ch)
 
         case '/':
             lnav_data.ld_mode = LNM_SEARCH;
-            lnav_data.ld_previous_search = lnav_data.ld_last_search[tc - lnav_data.ld_views];
+            lnav_data.ld_previous_search = tc->get_last_search();
             lnav_data.ld_search_start_line = tc->get_top();
-            add_view_text_possibilities(LNM_SEARCH, "*", tc);
+            add_view_text_possibilities(lnav_data.ld_rl_view, LNM_SEARCH, "*", tc);
             lnav_data.ld_rl_view->focus(LNM_SEARCH, "/");
+            lnav_data.ld_doc_status_source.set_title("Syntax Help");
+            rl_set_help();
             lnav_data.ld_bottom_source.set_prompt(
                     "Enter a regular expression to search for: "
                             "(Press " ANSI_BOLD("CTRL+]") " to abort)");
@@ -775,25 +796,28 @@ void handle_paging_key(int ch)
                 tc == &lnav_data.ld_views[LNV_DB] ||
                 tc == &lnav_data.ld_views[LNV_SCHEMA]) {
                 textview_curses &log_view = lnav_data.ld_views[LNV_LOG];
+
                 lnav_data.ld_exec_context.ec_top_line = tc->get_top();
 
                 lnav_data.ld_mode = LNM_SQL;
                 setup_logline_table(lnav_data.ld_exec_context);
                 lnav_data.ld_rl_view->focus(LNM_SQL, ";");
 
+                lnav_data.ld_doc_status_source.set_title("Query Help");
+                rl_set_help();
                 lnav_data.ld_bottom_source.update_loading(0, 0);
                 lnav_data.ld_status[LNS_BOTTOM].do_update();
 
                 field_overlay_source *fos;
 
-                fos = (field_overlay_source *)log_view.get_overlay_source();
+                fos = (field_overlay_source *) log_view.get_overlay_source();
                 fos->fos_active_prev = fos->fos_active;
                 if (!fos->fos_active) {
                     fos->fos_active = true;
                     tc->reload_data();
                 }
-                lnav_data.ld_bottom_source.set_prompt("Enter an SQL query: (Press "
-                ANSI_BOLD("CTRL+]") " to abort)");
+                lnav_data.ld_bottom_source.set_prompt(
+                    "Enter an SQL query: (Press " ANSI_BOLD("CTRL+]") " to abort)");
             }
             break;
 
@@ -808,7 +832,7 @@ void handle_paging_key(int ch)
             for (const auto &iter : scripts) {
                 lnav_data.ld_rl_view->add_possibility(LNM_EXEC, "__command", iter.first);
             }
-            add_view_text_possibilities(LNM_EXEC, "*", tc);
+            add_view_text_possibilities(lnav_data.ld_rl_view, LNM_EXEC, "*", tc);
             add_env_possibilities(LNM_EXEC);
             lnav_data.ld_rl_view->focus(LNM_EXEC, "|");
             lnav_data.ld_bottom_source.set_prompt(
@@ -834,7 +858,7 @@ void handle_paging_key(int ch)
             break;
 
         case 't':
-            if (lnav_data.ld_text_source.current_file() == NULL) {
+            if (lnav_data.ld_text_source.current_file() == nullptr) {
                 alerter::singleton().chime();
                 lnav_data.ld_rl_view->set_value("No text files loaded");
             }
@@ -856,19 +880,20 @@ void handle_paging_key(int ch)
 
         case 'I':
         {
-            time_t log_top = lnav_data.ld_top_time;
             hist_source2 &hs = lnav_data.ld_hist_source2;
 
             if (toggle_view(&lnav_data.ld_views[LNV_HISTOGRAM])) {
-                tc = lnav_data.ld_view_stack.back();
+                struct timeval log_top = lss->time_for_row(lnav_data.ld_views[LNV_LOG].get_top());
+
+                tc = *lnav_data.ld_view_stack.top();
                 tc->set_top(vis_line_t(hs.row_for_time(log_top)));
             }
             else {
                 textview_curses &hist_tc = lnav_data.ld_views[LNV_HISTOGRAM];
                 textview_curses &log_tc = lnav_data.ld_views[LNV_LOG];
                 lss = &lnav_data.ld_log_source;
-                time_t hist_top_time = hs.time_for_row(hist_tc.get_top());
-                time_t curr_top_time = lss->time_for_row(log_tc.get_top());
+                struct timeval hist_top_time = hs.time_for_row(hist_tc.get_top());
+                struct timeval curr_top_time = lss->time_for_row(log_tc.get_top());
                 if (hs.row_for_time(hist_top_time) != hs.row_for_time(curr_top_time)) {
                     vis_line_t new_top = lss->find_from_time(hist_top_time);
                     log_tc.set_top(new_top);
@@ -978,6 +1003,11 @@ void handle_paging_key(int ch)
                 );
 
                 tc->reload_data();
+            } else if (tc_tss != nullptr && tc_tss->tss_supports_filtering) {
+                lnav_data.ld_mode = LNM_FILTER;
+                lnav_data.ld_filter_view.reload_data();
+            } else {
+                alerter::singleton().chime();
             }
             break;
 
@@ -990,22 +1020,6 @@ void handle_paging_key(int ch)
             tc->set_needs_update();
             break;
 
-        case '\\':
-        {
-            string ex;
-
-            for (auto &iter : bm[&BM_EXAMPLE]) {
-                string line;
-
-                tc->get_sub_source()->text_value_for_line(*tc, iter, line);
-                ex += line + "\n";
-            }
-            lnav_data.ld_views[LNV_EXAMPLE].set_sub_source(new plain_text_source(
-                    ex));
-            ensure_view(&lnav_data.ld_views[LNV_EXAMPLE]);
-        }
-            break;
-
         case 'r':
         case 'R':
             if (lss) {
