@@ -29,7 +29,6 @@
 
 #include "config.h"
 
-#include <math.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
@@ -67,6 +66,7 @@ string_attr_type logline::L_FILE("file");
 string_attr_type logline::L_PARTITION("partition");
 string_attr_type logline::L_MODULE("module");
 string_attr_type logline::L_OPID("opid");
+string_attr_type logline::L_META("meta");
 
 external_log_format::mod_map_t external_log_format::MODULE_FORMATS;
 std::vector<external_log_format *> external_log_format::GRAPH_ORDERED_FORMATS;
@@ -169,37 +169,38 @@ const char *log_format::log_scanf(const char *line,
                                   struct timeval *tv_out,
                                   ...)
 {
-    int     curr_fmt = -1;
-    const char *  retval   = NULL;
+    int curr_fmt = -1;
+    const char *retval = NULL;
     bool done = false;
     pcre_input pi(line, 0, len);
     pcre_context_static<128> pc;
     va_list args;
+    int pat_index = this->last_pattern_index();
 
-    while (!done && next_format(fmt, curr_fmt, this->lf_fmt_lock)) {
+    while (!done && next_format(fmt, curr_fmt, pat_index)) {
         va_start(args, tv_out);
 
         pi.reset(line, 0, len);
-        if (!fmt[curr_fmt].pcre.match(pc, pi)) {
+        if (!fmt[curr_fmt].pcre.match(pc, pi, PCRE_NO_UTF8_CHECK)) {
             retval = NULL;
         }
         else {
-            pcre_context::capture_t *ts = pc["timestamp"];
+            pcre_context::capture_t *ts = pc[fmt[curr_fmt].pf_timestamp_index];
 
-            for (pcre_context::iterator iter = pc.begin();
-                 iter != pc.end();
-                 ++iter) {
+            for (auto &iter : pc) {
                 pcre_context::capture_t *cap = va_arg(
                         args, pcre_context::capture_t *);
 
-                *cap = *iter;
+                *cap = iter;
             }
 
             retval = this->lf_date_time.scan(
                     pi.get_substr_start(ts), ts->length(), NULL, tm_out, *tv_out);
 
             if (retval) {
-                this->lf_fmt_lock = curr_fmt;
+                if (curr_fmt != pat_index) {
+                    this->lf_pattern_locks.emplace_back(0, curr_fmt);
+                }
                 this->lf_timestamp_flags = tm_out->et_flags;
                 done = true;
             }
@@ -369,20 +370,11 @@ static int json_array_start(void *ctx)
 {
     yajlpp_parse_context *ypc = (yajlpp_parse_context *)ctx;
     json_log_userdata *jlu = (json_log_userdata *)ypc->ypc_userdata;
-    vector<external_log_format::json_format_element> &line_format =
-        jlu->jlu_format->jlf_line_format;
 
     if (ypc->ypc_path_index_stack.size() == 2) {
         const intern_string_t field_name = ypc->get_path_fragment_i(0);
 
-        if (!jlu->jlu_format->jlf_hide_extra &&
-            find_if(line_format.begin(), line_format.end(),
-                    external_log_format::json_field_cmp(
-                        external_log_format::JLF_VARIABLE,
-                        field_name)) == line_format.end()) {
-            jlu->jlu_sub_line_count += 1;
-        }
-
+        jlu->jlu_sub_line_count += jlu->jlu_format->value_line_count(field_name, true);
         jlu->jlu_sub_start = yajl_get_bytes_consumed(jlu->jlu_handle) - 1;
     }
 
@@ -430,7 +422,7 @@ static int rewrite_json_null(yajlpp_parse_context *ypc)
     if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
         return 1;
     }
-    jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name));
+    jlu->jlu_format->jlf_line_values.emplace_back(field_name);
 
     return 1;
 }
@@ -443,7 +435,7 @@ static int rewrite_json_bool(yajlpp_parse_context *ypc, int val)
     if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
         return 1;
     }
-    jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name, (bool)val));
+    jlu->jlu_format->jlf_line_values.emplace_back(field_name, (bool)val);
 
     return 1;
 }
@@ -456,7 +448,7 @@ static int rewrite_json_int(yajlpp_parse_context *ypc, long long val)
     if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
         return 1;
     }
-    jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name, (int64_t)val));
+    jlu->jlu_format->jlf_line_values.emplace_back(field_name, (int64_t)val);
 
     return 1;
 }
@@ -469,7 +461,7 @@ static int rewrite_json_double(yajlpp_parse_context *ypc, double val)
     if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
         return 1;
     }
-    jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name, val));
+    jlu->jlu_format->jlf_line_values.emplace_back(field_name, val);
 
     return 1;
 }
@@ -491,7 +483,7 @@ bool external_log_format::scan_for_partial(shared_buffer_ref &sbr, size_t &len_o
         return false;
     }
 
-    auto pat = this->elf_pattern_order[this->lf_fmt_lock];
+    auto pat = this->elf_pattern_order[this->last_pattern_index()];
     pcre_input pi(sbr.get_data(), 0, sbr.length());
 
     if (!this->elf_multiline) {
@@ -523,9 +515,12 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
             return log_format::SCAN_INCOMPLETE;
         }
 
-        const unsigned char *line_data = (const unsigned char *) sbr.get_data();
-        size_t line_end = sbr.length() - 1;
+        const auto *line_data = (const unsigned char *) sbr.get_data();
+        size_t line_end = sbr.length();
 
+        if (line_end > 0) {
+            line_end -= 1;
+        }
         while (line_end > 0 && isspace(line_data[line_end])) {
             line_end -= 1;
         }
@@ -578,9 +573,10 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
 
     pcre_input pi(sbr.get_data(), 0, sbr.length());
     pcre_context_static<128> pc;
-    int curr_fmt = -1;
+    int curr_fmt = -1, orig_lock = this->last_pattern_index();
+    int pat_index = orig_lock;
 
-    while (::next_format(this->elf_pattern_order, curr_fmt, this->lf_fmt_lock)) {
+    while (::next_format(this->elf_pattern_order, curr_fmt, pat_index)) {
         auto fpat = this->elf_pattern_order[curr_fmt];
         pcrepp *pat = fpat->p_pcre;
 
@@ -589,6 +585,11 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
         }
 
         if (!pat->match(pc, pi)) {
+            if (!this->lf_pattern_locks.empty() && pat_index != -1) {
+                log_debug("no match on pattern %d", pat_index);
+                curr_fmt = -1;
+                pat_index = -1;
+            }
             continue;
         }
 
@@ -628,14 +629,14 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
             this->check_for_new_year(dst, log_time_tm, log_tv);
         }
 
-        if (opid_cap != NULL) {
+        if (opid_cap != nullptr) {
             opid = hash_str(pi.get_substr_start(opid_cap), opid_cap->length());
         }
 
-        if (mod_cap != NULL) {
+        if (mod_cap != nullptr) {
             intern_string_t mod_name = intern_string::lookup(
                     pi.get_substr_start(mod_cap), mod_cap->length());
-            mod_map_t::iterator mod_iter = MODULE_FORMATS.find(mod_name);
+            auto mod_iter = MODULE_FORMATS.find(mod_name);
 
             if (mod_iter == MODULE_FORMATS.end()) {
                 mod_index = module_scan(pi, body_cap, mod_name);
@@ -650,13 +651,13 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
             const value_def &vd = *ivd.ivd_value_def;
             pcre_context::capture_t *num_cap = pc[ivd.ivd_index];
 
-            if (num_cap != NULL && num_cap->is_valid()) {
-                const struct scaling_factor *scaling = NULL;
+            if (num_cap != nullptr && num_cap->is_valid()) {
+                const struct scaling_factor *scaling = nullptr;
 
                 if (ivd.ivd_unit_field_index >= 0) {
                     pcre_context::iterator unit_cap = pc[ivd.ivd_unit_field_index];
 
-                    if (unit_cap != NULL && unit_cap->is_valid()) {
+                    if (unit_cap != nullptr && unit_cap->is_valid()) {
                         intern_string_t unit_val = intern_string::lookup(
                             pi.get_substr_start(unit_cap), unit_cap->length());
                         std::map<const intern_string_t, scaling_factor>::const_iterator unit_iter;
@@ -670,12 +671,12 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
                     }
                 }
 
-                char cap_copy[num_cap->length() + 1];
-                double dvalue;
+                const char *num_cap_start = pi.get_substr_start(num_cap);
+                const char *num_cap_end = num_cap_start + num_cap->length();
+                double dvalue = strtod(num_cap_start, (char **) &num_cap_end);
 
-                pi.get_substr(num_cap, cap_copy);
-                if (sscanf(cap_copy, "%lf", &dvalue) == 1) {
-                    if (scaling != NULL) {
+                if (num_cap_end == num_cap_start + num_cap->length()) {
+                    if (scaling != nullptr) {
                         scaling->scale(dvalue);
                     }
                     this->lf_value_stats[vd.vd_values_index].add_value(dvalue);
@@ -685,7 +686,10 @@ log_format::scan_result_t external_log_format::scan(nonstd::optional<logfile *>
 
         dst.emplace_back(offset, log_tv, level, mod_index, opid);
 
-        this->lf_fmt_lock = curr_fmt;
+        if (orig_lock != curr_fmt) {
+            log_debug("changing pattern lock %d -> %d", orig_lock, curr_fmt);
+            this->lf_pattern_locks.emplace_back(dst.size() - 1, curr_fmt);
+        }
         return log_format::SCAN_MATCH;
     }
 
@@ -703,10 +707,7 @@ uint8_t external_log_format::module_scan(const pcre_input &pi,
     pcre_context_static<128> pc;
     module_format mf;
 
-    for (vector<external_log_format *>::iterator fmt_iter = ext_fmts.begin();
-         fmt_iter != ext_fmts.end();
-         ++fmt_iter) {
-        external_log_format *elf = *fmt_iter;
+    for (auto elf : ext_fmts) {
         int curr_fmt = -1, fmt_lock = -1;
 
         while (::next_format(elf->elf_pattern_order, curr_fmt, fmt_lock)) {
@@ -739,10 +740,8 @@ uint8_t external_log_format::module_scan(const pcre_input &pi,
     return 0;
 }
 
-void external_log_format::annotate(shared_buffer_ref &line,
-                                   string_attrs_t &sa,
-                                   std::vector<logline_value> &values,
-                                   bool annotate_module) const
+void external_log_format::annotate(uint64_t line_number, shared_buffer_ref &line, string_attrs_t &sa,
+                                   std::vector<logline_value> &values, bool annotate_module) const
 {
     pcre_context_static<128> pc;
     pcre_input pi(line.get_data(), 0, line.length());
@@ -755,9 +754,14 @@ void external_log_format::annotate(shared_buffer_ref &line,
         return;
     }
 
-    pattern &pat = *this->elf_pattern_order[this->lf_fmt_lock];
+    int pat_index = this->pattern_index_for_line(line_number);
+    pattern &pat = *this->elf_pattern_order[pat_index];
 
     if (!pat.p_pcre->match(pc, pi)) {
+        // A continued line still needs a body.
+        lr.lr_start = 0;
+        lr.lr_end = line.length();
+        sa.emplace_back(lr, &textview_curses::SA_BODY);
         return;
     }
 
@@ -821,16 +825,16 @@ void external_log_format::annotate(shared_buffer_ref &line,
 
         field.subset(line, cap->c_begin, cap->length());
 
-        values.push_back(logline_value(vd.vd_name,
-                                       vd.vd_kind,
-                                       field,
-                                       vd.vd_identifier,
-                                       scaling,
-                                       vd.vd_column,
-                                       cap->c_begin,
-                                       cap->c_end,
-                                       pat.p_module_format,
-                                       this));
+        values.emplace_back(vd.vd_name,
+                            vd.vd_kind,
+                            field,
+                            vd.vd_identifier,
+                            scaling,
+                            vd.vd_column,
+                            cap->c_begin,
+                            cap->c_end,
+                            pat.p_module_format,
+                            this);
         values.back().lv_hidden = vd.vd_hidden || vd.vd_user_hidden;
     }
 
@@ -838,7 +842,7 @@ void external_log_format::annotate(shared_buffer_ref &line,
             body_cap->is_valid()) {
         intern_string_t mod_name = intern_string::lookup(
                 pi.get_substr_start(module_cap), module_cap->length());
-        mod_map_t::iterator mod_iter = MODULE_FORMATS.find(mod_name);
+        auto mod_iter = MODULE_FORMATS.find(mod_name);
 
         if (mod_iter != MODULE_FORMATS.end() &&
                 mod_iter->second.mf_mod_format != NULL) {
@@ -847,15 +851,13 @@ void external_log_format::annotate(shared_buffer_ref &line,
 
             body_cap->ltrim(line.get_data());
             body_ref.subset(line, body_cap->c_begin, body_cap->length());
-            mf.mf_mod_format->annotate(body_ref, sa, values, false);
-            for (vector<logline_value>::iterator lv_iter = values.begin();
-                 lv_iter != values.end();
-                 ++lv_iter) {
-                if (!lv_iter->lv_from_module) {
+            mf.mf_mod_format->annotate(line_number, body_ref, sa, values, false);
+            for (auto &value : values) {
+                if (!value.lv_from_module) {
                     continue;
                 }
-                lv_iter->lv_origin.lr_start += body_cap->c_begin;
-                lv_iter->lv_origin.lr_end += body_cap->c_begin;
+                value.lv_origin.lr_start += body_cap->c_begin;
+                value.lv_origin.lr_end += body_cap->c_begin;
             }
         }
     }
@@ -866,7 +868,7 @@ void external_log_format::rewrite(exec_context &ec,
                                   string_attrs_t &sa,
                                   string &value_out)
 {
-    vector<logline_value>::iterator iter, bind_iter, shift_iter;
+    vector<logline_value>::iterator iter, shift_iter;
     vector<logline_value> &values = *ec.ec_line_values;
 
     value_out.assign(line.get_data(), line.length());
@@ -963,7 +965,7 @@ static int rewrite_json_field(yajlpp_parse_context *ypc, const unsigned char *st
                          jlu->jlu_line->get_timeval(), 'T');
         }
         tmp_shared_buffer tsb(time_buf);
-        jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name, tsb.tsb_ref));
+        jlu->jlu_format->jlf_line_values.emplace_back(field_name, tsb.tsb_ref);
     }
     else if (jlu->jlu_shared_buffer.contains((const char *)str)) {
         shared_buffer_ref sbr;
@@ -972,27 +974,25 @@ static int rewrite_json_field(yajlpp_parse_context *ypc, const unsigned char *st
                 (off_t) ((const char *)str - jlu->jlu_line_value),
                 len);
         if (field_name == jlu->jlu_format->elf_body_field) {
-            jlu->jlu_format->jlf_line_values.push_back(logline_value(body_name,
-                    sbr));
+            jlu->jlu_format->jlf_line_values.emplace_back(body_name, sbr);
         }
         if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
             return 1;
         }
 
-        jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name,
-                sbr));
+        jlu->jlu_format->jlf_line_values.emplace_back(field_name, sbr);
     }
     else {
         tmp_shared_buffer tsb((const char *)str, len);
 
         if (field_name == jlu->jlu_format->elf_body_field) {
-            jlu->jlu_format->jlf_line_values.push_back(logline_value(body_name, tsb.tsb_ref));
+            jlu->jlu_format->jlf_line_values.emplace_back(body_name, tsb.tsb_ref);
         }
         if (!ypc->is_level(1) && !jlu->jlu_format->has_value_def(field_name)) {
             return 1;
         }
 
-        jlu->jlu_format->jlf_line_values.push_back(logline_value(field_name, tsb.tsb_ref));
+        jlu->jlu_format->jlf_line_values.emplace_back(field_name, tsb.tsb_ref);
     }
 
     return 1;
@@ -1004,7 +1004,8 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
         return;
     }
 
-    if (this->jlf_cached_offset != ll.get_offset()) {
+    if (this->jlf_cached_offset != ll.get_offset() ||
+        this->jlf_cached_full != full_message) {
         yajlpp_parse_context &ypc = *(this->jlf_parse_context);
         yajl_handle handle = this->jlf_yajl_handle.in();
         json_log_userdata jlu(sbr);
@@ -1033,7 +1034,6 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
         if (parse_status == yajl_status_ok &&
             yajl_complete_parse(handle) == yajl_status_ok) {
             std::vector<logline_value>::iterator lv_iter;
-            std::vector<json_format_element>::iterator iter;
             bool used_values[this->jlf_line_values.size()];
             struct line_range lr;
 
@@ -1110,10 +1110,9 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
                             this->json_append(jfe, str.c_str(), str.size());
                         }
 
-                        if (nl_pos == string::npos) {
+                        if (nl_pos == string::npos || full_message) {
                             lr.lr_end = this->jlf_cached_line.size();
-                        }
-                        else {
+                        } else {
                             lr.lr_end = lr.lr_start + nl_pos;
                         }
 
@@ -1244,6 +1243,7 @@ void external_log_format::get_subline(const logline &ll, shared_buffer_ref &sbr,
         }
 
         this->jlf_cached_offset = ll.get_offset();
+        this->jlf_cached_full = full_message;
     }
 
     off_t this_off = 0, next_off = 0;
@@ -1452,11 +1452,10 @@ void external_log_format::build(std::vector<std::string> &errors) {
         }
     }
 
-    for (std::map<log_level_t, level_pattern>::iterator iter = this->elf_level_patterns.begin();
-         iter != this->elf_level_patterns.end();
-         ++iter) {
+    for (auto &elf_level_pattern : this->elf_level_patterns) {
         try {
-            iter->second.lp_pcre = new pcrepp(iter->second.lp_regex.c_str());
+            elf_level_pattern.second.lp_pcre = new pcrepp(
+                elf_level_pattern.second.lp_regex.c_str());
         }
         catch (const pcrepp::error &e) {
             errors.push_back("error:" +
@@ -1466,27 +1465,26 @@ void external_log_format::build(std::vector<std::string> &errors) {
 
     stable_sort(this->elf_level_pairs.begin(), this->elf_level_pairs.end());
 
-    for (auto iter = this->elf_value_defs.begin();
-         iter != this->elf_value_defs.end();
-         ++iter) {
+    for (auto &elf_value_def : this->elf_value_defs) {
         std::vector<std::string>::iterator act_iter;
 
-        if (!iter->second->vd_internal && iter->second->vd_column == -1) {
-            iter->second->vd_column = this->elf_column_count++;
+        if (!elf_value_def.second->vd_internal &&
+            elf_value_def.second->vd_column == -1) {
+            elf_value_def.second->vd_column = this->elf_column_count++;
         }
 
-        if (iter->second->vd_kind == logline_value::VALUE_UNKNOWN) {
-            iter->second->vd_kind = logline_value::VALUE_TEXT;
+        if (elf_value_def.second->vd_kind == logline_value::VALUE_UNKNOWN) {
+            elf_value_def.second->vd_kind = logline_value::VALUE_TEXT;
         }
 
-        for (act_iter = iter->second->vd_action_list.begin();
-             act_iter != iter->second->vd_action_list.end();
+        for (act_iter = elf_value_def.second->vd_action_list.begin();
+             act_iter != elf_value_def.second->vd_action_list.end();
              ++act_iter) {
             if (this->lf_action_defs.find(*act_iter) ==
                 this->lf_action_defs.end()) {
                 errors.push_back("error:" +
                                  this->elf_name.to_string() + ":" +
-                                 iter->first.get() +
+                                     elf_value_def.first.get() +
                                  ": cannot find action -- " + (*act_iter));
             }
         }
@@ -1651,7 +1649,7 @@ void external_log_format::build(std::vector<std::string> &errors) {
     this->lf_value_stats.resize(this->elf_numeric_value_defs.size());
 
     int format_index = 0;
-    for (vector<json_format_element>::iterator iter = this->jlf_line_format.begin();
+    for (auto iter = this->jlf_line_format.begin();
          iter != this->jlf_line_format.end();
          ++iter, format_index++) {
         static const intern_string_t ts = intern_string::lookup("__timestamp__");
@@ -1832,14 +1830,15 @@ public:
         cols.resize(elf.elf_column_count);
         for (const auto &elf_value_def : elf.elf_value_defs) {
             const auto &vd = *elf_value_def.second;
-            int type = log_vtab_impl::logline_value_to_sqlite_type(vd.vd_kind);
+            pair<int, unsigned int> type_pair = log_vtab_impl::logline_value_to_sqlite_type(vd.vd_kind);
 
             if (vd.vd_column == -1) {
                 continue;
             }
 
             cols[vd.vd_column].vc_name = vd.vd_name.get();
-            cols[vd.vd_column].vc_type = type;
+            cols[vd.vd_column].vc_type = type_pair.first;
+            cols[vd.vd_column].vc_subtype = type_pair.second;
             cols[vd.vd_column].vc_collator = vd.vd_collate.c_str();
             cols[vd.vd_column].vc_comment = vd.vd_description;
         }
@@ -1876,7 +1875,7 @@ public:
 
         log_format *format = lf->get_format();
 
-        this->elt_module_format.mf_mod_format = NULL;
+        this->elt_module_format.mf_mod_format = nullptr;
         if (format->get_name() == this->lfvi_format.get_name()) {
             return true;
         } else if (mod_id && mod_id == this->lfvi_format.lf_mod_index) {
@@ -1888,7 +1887,7 @@ public:
 
             lf->read_line(lf_iter, line);
             this->vi_attrs.clear();
-            format->annotate(line, this->vi_attrs, values, false);
+            format->annotate(cl, line, this->vi_attrs, values, false);
             this->elt_container_body = find_string_attr_range(this->vi_attrs, &textview_curses::SA_BODY);
             if (!this->elt_container_body.is_valid()) {
                 return false;
@@ -1918,23 +1917,28 @@ public:
     };
 
     virtual void extract(shared_ptr<logfile> lf,
+                         uint64_t line_number,
                          shared_buffer_ref &line,
                          std::vector<logline_value> &values)
     {
         log_format *format = lf->get_format();
 
-        if (this->elt_module_format.mf_mod_format != NULL) {
+        if (this->elt_module_format.mf_mod_format != nullptr) {
             shared_buffer_ref body_ref;
 
             body_ref.subset(line, this->elt_container_body.lr_start,
                             this->elt_container_body.length());
             this->vi_attrs.clear();
             values.clear();
-            this->elt_module_format.mf_mod_format->annotate(body_ref, this->vi_attrs, values);
+            this->elt_module_format.mf_mod_format->annotate(line_number,
+                                                            body_ref,
+                                                            this->vi_attrs,
+                                                            values,
+                                                            false);
         }
         else {
             this->vi_attrs.clear();
-            format->annotate(line, this->vi_attrs, values, false);
+            format->annotate(line_number, line, this->vi_attrs, values, false);
         }
     };
 
@@ -1943,10 +1947,33 @@ public:
     struct line_range elt_container_body;
 };
 
-log_vtab_impl *external_log_format::get_vtab_impl(void) const
+log_vtab_impl *external_log_format::get_vtab_impl() const
 {
     return new external_log_table(*this);
 }
 
+int log_format::pattern_index_for_line(uint64_t line_number) const
+{
+    auto iter = lower_bound(this->lf_pattern_locks.cbegin(),
+                            this->lf_pattern_locks.cend(),
+                            line_number,
+                            [](const pattern_for_lines &pfl, uint32_t line) {
+        return pfl.pfl_line < line;
+    });
+
+    if (iter == this->lf_pattern_locks.end() ||
+        iter->pfl_line != line_number) {
+        --iter;
+    }
+
+    return iter->pfl_pat_index;
+}
+
+log_format::pattern_for_lines::pattern_for_lines(
+    uint32_t pfl_line, uint32_t pfl_pat_index) :
+    pfl_line(pfl_line), pfl_pat_index(pfl_pat_index)
+{
+}
+
 /* XXX */
 #include "log_format_impls.cc"
