@@ -111,8 +111,9 @@ class generic_log_format : public log_format {
         return log_fmt;
     };
 
-    std::string get_pattern_regex() const {
-        return get_pcre_log_formats()[this->lf_fmt_lock].name;
+    std::string get_pattern_regex(uint64_t line_number) const {
+        int pat_index = this->pattern_index_for_line(line_number);
+        return get_pcre_log_formats()[pat_index].name;
     }
 
     intern_string_t get_name() const {
@@ -150,12 +151,12 @@ class generic_log_format : public log_format {
                 sbr.get_data(),
                 sbr.length(),
                 get_pcre_log_formats(),
-                NULL,
+                nullptr,
                 &log_time,
                 &log_tv,
 
                 &ts,
-                &level)) != NULL) {
+                &level)) != nullptr) {
             const char *level_str = &sbr.get_data()[level.c_begin];
             log_level_t level_val = string2level(level_str, level.length());
 
@@ -165,19 +166,18 @@ class generic_log_format : public log_format {
                 this->check_for_new_year(dst, log_time, log_tv);
             }
 
-            dst.push_back(logline(offset, log_tv, level_val));
+            dst.emplace_back(offset, log_tv, level_val);
             return SCAN_MATCH;
         }
 
         return SCAN_NO_MATCH;
     };
 
-    void annotate(shared_buffer_ref &line,
-                  string_attrs_t &sa,
-                  std::vector<logline_value> &values,
-                  bool annotate_module) const
+    void annotate(uint64_t line_number, shared_buffer_ref &line, string_attrs_t &sa,
+                      std::vector<logline_value> &values, bool annotate_module) const
     {
-        pcre_format &fmt = get_pcre_log_formats()[this->lf_fmt_lock];
+        int pat_index = this->pattern_index_for_line(line_number);
+        pcre_format &fmt = get_pcre_log_formats()[pat_index];
         struct line_range lr;
         int prefix_len = 0;
         pcre_input pi(line.get_data(), 0, line.length());
@@ -602,9 +602,8 @@ public:
         return SCAN_NO_MATCH;
     };
 
-    void annotate(shared_buffer_ref &sbr, string_attrs_t &sa,
-                  std::vector<logline_value> &values,
-                  bool annotate_module) const {
+    void annotate(uint64_t line_number, shared_buffer_ref &sbr, string_attrs_t &sa,
+                      std::vector<logline_value> &values, bool annotate_module) const {
         static const intern_string_t TS = intern_string::lookup("bro_ts");
         static const intern_string_t UID = intern_string::lookup("bro_uid");
 
@@ -676,9 +675,9 @@ public:
 
         void get_columns(vector<vtab_column> &cols) const {
             for (const auto &fd : this->blt_format.blf_field_defs) {
-                int type = log_vtab_impl::logline_value_to_sqlite_type(fd.fd_kind);
+                std::pair<int, unsigned int> type_pair = log_vtab_impl::logline_value_to_sqlite_type(fd.fd_kind);
 
-                cols.emplace_back(fd.fd_name.to_string(), type, fd.fd_collator);
+                cols.emplace_back(fd.fd_name.to_string(), type_pair.first, fd.fd_collator, false, "", type_pair.second);
             }
         };
 
