@@ -32,9 +32,11 @@
 #include "lnav.hh"
 #include "lnav_util.hh"
 #include "sysclip.hh"
+#include "vtab_module.hh"
 #include "plain_text_source.hh"
 #include "command_executor.hh"
 #include "readline_curses.hh"
+#include "readline_highlighters.hh"
 #include "log_search_table.hh"
 #include "log_format_loader.hh"
 #include "help_text_formatter.hh"
@@ -44,11 +46,103 @@ using namespace std;
 
 #define ABORT_MSG "(Press " ANSI_BOLD("CTRL+]") " to abort)"
 
+#define STR_HELPER(x) #x
+#define STR(x) STR_HELPER(x)
+
+#define ANSI_RE(msg) ANSI_CSI "1;3" STR(COLOR_CYAN) "m" msg ANSI_NORM
+#define ANSI_CLS(msg) ANSI_CSI "1;3" STR(COLOR_MAGENTA) "m" msg ANSI_NORM
+#define ANSI_KW(msg) ANSI_CSI "3" STR(COLOR_BLUE) "m" msg ANSI_NORM
+#define ANSI_REV(msg) ANSI_CSI "7m" msg ANSI_NORM
+#define ANSI_STR(msg) ANSI_CSI "32m" msg ANSI_NORM
+
+const char *RE_HELP =
+    " "  ANSI_RE(".") "   Any character    "
+    " "     "a" ANSI_RE("|") "b  a or b        "
+    " "                 ANSI_RE("^") "       Start of string\n"
+
+    " " ANSI_CLS("\\w") "  Word character   "
+    " "     "a" ANSI_RE("?") "   0 or 1 a's    "
+    " "                 ANSI_RE("$") "       End of string\n"
+
+    " " ANSI_CLS("\\d") "  Digit            "
+    " "     "a" ANSI_RE("*") "   0 or more a's "
+    " " ANSI_RE("(") "..." ANSI_RE(")") "   Capture\n"
+
+    " " ANSI_RE("\\") "   Escape character "
+    " "     "a" ANSI_RE("+") "   1 or more a's "
+    " " ANSI_RE("[") "ab" ANSI_RE("-") "d" ANSI_RE("]") "  Any of a, b, c, or d"
+;
+
+const char *RE_EXAMPLE =
+    ANSI_UNDERLINE("Examples") "\n"
+    "  abc" ANSI_RE("*") " matches "
+    ANSI_STR("'ab'") ", " ANSI_STR("'abc'") ", " ANSI_STR("'abccc'") "\n"
+
+    "  key=" ANSI_RE("(\\w+)")
+    " matches key=" ANSI_REV("123") ", key=" ANSI_REV("abc") " and captures 123 and abc\n"
+
+    "  " ANSI_RE("\\") "[abc" ANSI_RE("\\") "] matches " ANSI_STR("'[abc]'")
+;
+
+const char *SQL_HELP =
+    " " ANSI_KW("SELECT") "  Select rows from a table      "
+    " " ANSI_KW("DELETE") "  Delete rows from a table\n"
+    " " ANSI_KW("INSERT") "  Insert rows into a table      "
+    " " ANSI_KW("UPDATE") "  Update rows in a table\n"
+    " " ANSI_KW("CREATE") "  Create a table/index          "
+    " " ANSI_KW("DROP") "    Drop a table/index\n"
+    " " ANSI_KW("ATTACH") "  Attach a SQLite database file "
+    " " ANSI_KW("DETACH") "  Detach a SQLite database"
+;
+
+const char *SQL_EXAMPLE =
+    ANSI_UNDERLINE("Examples") "\n"
+    "  SELECT * FROM %s WHERE log_level >= 'warning' LIMIT 10\n"
+    "  UPDATE %s SET log_mark = 1 WHERE log_line = log_top_line()\n"
+    "  SELECT * FROM logline LIMIT 10"
+;
+
 static const char *LNAV_CMD_PROMPT = "Enter an lnav command: " ABORT_MSG;
 
+void rl_set_help()
+{
+    switch (lnav_data.ld_mode) {
+        case LNM_SEARCH: {
+            lnav_data.ld_doc_source.replace_with(RE_HELP);
+            lnav_data.ld_example_source.replace_with(RE_EXAMPLE);
+            break;
+        }
+        case LNM_SQL: {
+            textview_curses &log_view = lnav_data.ld_views[LNV_LOG];
+            auto lss = (logfile_sub_source *) log_view.get_sub_source();
+            char example_txt[1024];
+            attr_line_t example_al;
+
+            if (log_view.get_inner_height() > 0) {
+                auto cl = lss->at(log_view.get_top());
+                auto lf = lss->find(cl);
+                auto format_name = lf->get_format()->get_name().get();
+
+                snprintf(example_txt, sizeof(example_txt),
+                         SQL_EXAMPLE,
+                         format_name,
+                         format_name);
+                example_al.with_ansi_string(example_txt);
+                readline_sqlite_highlighter(example_al, 0);
+            }
+
+            lnav_data.ld_doc_source.replace_with(SQL_HELP);
+            lnav_data.ld_example_source.replace_with(example_al);
+            break;
+        }
+        default:
+            break;
+    }
+}
+
 void rl_change(void *dummy, readline_curses *rc)
 {
-    textview_curses *tc = lnav_data.ld_view_stack.back();
+    textview_curses *tc = *lnav_data.ld_view_stack.top();
 
     tc->get_highlights().erase("$preview");
     tc->get_highlights().erase("$bodypreview");
@@ -57,12 +151,24 @@ void rl_change(void *dummy, readline_curses *rc)
 
     switch (lnav_data.ld_mode) {
         case LNM_COMMAND: {
+            static string last_command;
+            static int generation = 0;
+
             string line = rc->get_line_buffer();
             vector<string> args;
-            readline_context::command_map_t::iterator iter = lnav_commands.end();
+            auto iter = lnav_commands.end();
 
             split_ws(line, args);
 
+            if (args.empty()) {
+                generation = 0;
+            } else if (args[0] != last_command) {
+                last_command = args[0];
+                generation = 0;
+            } else {
+                generation += 1;
+            }
+
             if (!args.empty()) {
                 iter = lnav_commands.find(args[0]);
             }
@@ -119,6 +225,16 @@ void rl_change(void *dummy, readline_curses *rc)
                     lnav_data.ld_example_source.replace_with(al);
                 }
 
+                if (cmd.c_prompt != nullptr && generation == 0 &&
+                    trim(line) == args[0]) {
+                    string new_prompt = cmd.c_prompt(
+                        lnav_data.ld_exec_context, line);
+
+                    if (!new_prompt.empty()) {
+                        rc->rewrite_line(line.length(), new_prompt);
+                    }
+                }
+
                 lnav_data.ld_bottom_source.grep_error("");
                 lnav_data.ld_status[LNS_BOTTOM].window_change();
             }
@@ -155,7 +271,7 @@ void rl_change(void *dummy, readline_curses *rc)
 
 static void rl_search_internal(void *dummy, readline_curses *rc, bool complete = false)
 {
-    textview_curses *tc = lnav_data.ld_view_stack.back();
+    textview_curses *tc = *lnav_data.ld_view_stack.top();
     string term_val;
     string name;
 
@@ -200,7 +316,7 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
     case LNM_SQL: {
         term_val = trim(rc->get_value() + ";");
 
-        if (term_val.size() > 0 && term_val[0] == '.') {
+        if (!term_val.empty() && term_val[0] == '.') {
             lnav_data.ld_bottom_source.grep_error("");
         } else if (!sqlite3_complete(term_val.c_str())) {
             lnav_data.ld_bottom_source.
@@ -240,7 +356,7 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
         }
 
         vector<string> kw;
-        auto iter = rfind_string_attr_if(sa, x, [&al, &name, &kw](auto sa) {
+        auto iter = rfind_string_attr_if(sa, x, [&al, &name, &kw, x](auto sa) {
             if (sa.sa_type != &SQL_FUNCTION_ATTR &&
                 sa.sa_type != &SQL_KEYWORD_ATTR) {
                 return false;
@@ -250,6 +366,12 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
             const line_range &lr = sa.sa_range;
             int lpc;
 
+            if (sa.sa_type == &SQL_FUNCTION_ATTR) {
+                if (!sa.sa_range.contains(x)) {
+                    return false;
+                }
+            }
+
             for (lpc = lr.lr_start; lpc < lr.lr_end; lpc++) {
                 if (!isalnum(str[lpc]) && str[lpc] != '_') {
                     break;
@@ -316,14 +438,16 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
         auto ident_iter = find_string_attr_containing(sa, &SQL_IDENTIFIER_ATTR, x);
         if (ident_iter != sa.end()) {
             string ident = al.get_substring(ident_iter->sa_range);
+            intern_string_t intern_ident = intern_string::lookup(ident);
 
-            auto vtab = lnav_data.ld_vtab_manager->lookup_impl(
-                intern_string::lookup(ident));
+            auto vtab = lnav_data.ld_vtab_manager->lookup_impl(intern_ident);
+            auto vtab_module_iter = vtab_module_ddls.find(intern_ident);
             string ddl;
 
             if (vtab != nullptr) {
                 ddl = trim(vtab->get_table_statement());
-
+            } else if (vtab_module_iter != vtab_module_ddls.end()) {
+                ddl = vtab_module_iter->second;
             } else {
                 auto table_ddl_iter = lnav_data.ld_table_ddl.find(ident);
 
@@ -343,8 +467,7 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
         }
 
         if (!has_doc) {
-            lnav_data.ld_doc_source.clear();
-            lnav_data.ld_example_source.clear();
+            rl_set_help();
             lnav_data.ld_preview_source.clear();
         }
         return;
@@ -358,17 +481,15 @@ static void rl_search_internal(void *dummy, readline_curses *rc, bool complete =
         break;
     }
 
-    lnav_view_t      index = (lnav_view_t)(tc - lnav_data.ld_views);
-
     if (!complete) {
         tc->set_top(lnav_data.ld_search_start_line);
     }
-    execute_search(index, rc->get_value());
+    tc->execute_search(rc->get_value());
 }
 
 void rl_search(void *dummy, readline_curses *rc)
 {
-    textview_curses *tc = lnav_data.ld_view_stack.back();
+    textview_curses *tc = *lnav_data.ld_view_stack.top();
 
     rl_search_internal(dummy, rc);
     tc->set_follow_search_for(0);
@@ -376,8 +497,7 @@ void rl_search(void *dummy, readline_curses *rc)
 
 void rl_abort(void *dummy, readline_curses *rc)
 {
-    textview_curses *tc    = lnav_data.ld_view_stack.back();
-    lnav_view_t      index = (lnav_view_t)(tc - lnav_data.ld_views);
+    textview_curses *tc = *lnav_data.ld_view_stack.top();
 
     lnav_data.ld_bottom_source.set_prompt("");
     lnav_data.ld_example_source.clear();
@@ -391,7 +511,7 @@ void rl_abort(void *dummy, readline_curses *rc)
     switch (lnav_data.ld_mode) {
     case LNM_SEARCH:
         tc->set_top(lnav_data.ld_search_start_line);
-        execute_search(index, lnav_data.ld_previous_search);
+        tc->execute_search(lnav_data.ld_previous_search);
         break;
     case LNM_SQL:
         tc->reload_data();
@@ -405,7 +525,7 @@ void rl_abort(void *dummy, readline_curses *rc)
 
 void rl_callback(void *dummy, readline_curses *rc)
 {
-    textview_curses *tc = lnav_data.ld_view_stack.back();
+    textview_curses *tc = *lnav_data.ld_view_stack.top();
     exec_context &ec = lnav_data.ld_exec_context;
     string alt_msg;
 
@@ -418,6 +538,7 @@ void rl_callback(void *dummy, readline_curses *rc)
     tc->get_highlights().erase("$bodypreview");
     switch (lnav_data.ld_mode) {
     case LNM_PAGING:
+    case LNM_FILTER:
         require(0);
         break;
 
@@ -431,13 +552,12 @@ void rl_callback(void *dummy, readline_curses *rc)
         rl_search_internal(dummy, rc, true);
         if (rc->get_value().size() > 0) {
             auto_mem<FILE> pfile(pclose);
-            textview_curses *tc = lnav_data.ld_view_stack.back();
             vis_bookmarks &bm = tc->get_bookmarks();
             const auto &bv = bm[&textview_curses::BM_SEARCH];
             vis_line_t vl = bv.next(tc->get_top());
 
             pfile = open_clipboard(CT_FIND);
-            if (pfile.in() != NULL) {
+            if (pfile.in() != nullptr) {
                 fprintf(pfile, "%s", rc->get_value().c_str());
             }
             if (vl != -1_vl) {
@@ -458,10 +578,9 @@ void rl_callback(void *dummy, readline_curses *rc)
 
         if (!result.empty()) {
             result = "SQL Result: " + result;
-        }
-
-        if (dls.dls_rows.size() > 1) {
-            ensure_view(&lnav_data.ld_views[LNV_DB]);
+            if (dls.dls_rows.size() > 1) {
+                ensure_view(&lnav_data.ld_views[LNV_DB]);
+            }
         }
 
         rc->set_value(result);
@@ -475,23 +594,23 @@ void rl_callback(void *dummy, readline_curses *rc)
         tmpout = std::tmpfile();
 
         if (!tmpout) {
-            rc->set_value("Unable to open temporary output file: " + string(strerror(errno)));
-        }
-        else {
+            rc->set_value("Unable to open temporary output file: " +
+                          string(strerror(errno)));
+        } else {
             auto_fd fd(fileno(tmpout));
             auto_fd fd_copy((const auto_fd &) fd);
             char desc[256], timestamp[32];
             time_t current_time = time(NULL);
             string path_and_args = rc->get_value();
 
-	    lnav_data.ld_output_stack.push(tmpout);
-	    string result = execute_file(ec, path_and_args);
-	    string::size_type lf_index = result.find('\n');
-	    if (lf_index != string::npos) {
-		result = result.substr(0, lf_index);
-	    }
-	    rc->set_value(result);
-	    lnav_data.ld_output_stack.pop();
+            ec.ec_output_stack.back() = tmpout.in();
+            string result = execute_file(ec, path_and_args);
+            string::size_type lf_index = result.find('\n');
+            if (lf_index != string::npos) {
+                result = result.substr(0, lf_index);
+            }
+            rc->set_value(result);
+            ec.ec_output_stack.back() = nonstd::nullopt;
 
             struct stat st;
 
@@ -506,11 +625,11 @@ void rl_callback(void *dummy, readline_curses *rc)
                 lnav_data.ld_file_names[desc]
                     .with_fd(fd_copy)
                     .with_detect_format(false);
-                lnav_data.ld_files_to_front.push_back(make_pair(desc, 0));
+                lnav_data.ld_files_to_front.emplace_back(desc, 0);
 
                 if (lnav_data.ld_rl_view != NULL) {
-                    lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(
-                                                            X, "to close the file"));
+                    lnav_data.ld_rl_view->set_alt_value(
+                        HELP_MSG_1(X, "to close the file"));
                 }
             }
         }
@@ -528,32 +647,43 @@ void rl_display_matches(void *dummy, readline_curses *rc)
     unsigned long width;
     __attribute((unused))
     unsigned long height;
-    int max_len, cols, rows;
+    int max_len, cols;
 
     getmaxyx(lnav_data.ld_window, height, width);
 
     max_len = rc->get_max_match_length() + 2;
     cols = max(1UL, width / max_len);
-    rows = (matches.size() + cols - 1) / cols;
 
     if (matches.empty()) {
         lnav_data.ld_match_source.clear();
     }
-    else if (cols == 1) {
-        lnav_data.ld_match_source.replace_with(rc->get_matches());
-    }
     else {
-        std::vector<std::string> horiz_matches;
+        string current_match = rc->get_match_string();
+        int curr_col = 0;
+        attr_line_t al;
+        bool add_nl = false;
 
-        horiz_matches.resize(rows);
-        for (size_t lpc = 0; lpc < matches.size(); lpc++) {
-            int curr_row = lpc % rows;
+        for (const auto &match : matches) {
+            if (add_nl) {
+                al.append(1, '\n');
+                add_nl = false;
+            }
+            if (match == current_match) {
+                al.append(match, &view_curses::VC_STYLE, A_REVERSE);
+            } else {
+                al.append(match);
+            }
+            curr_col += 1;
+            if (curr_col < cols) {
+                int padding = max_len - match.size();
 
-            horiz_matches[curr_row].append(matches[lpc]);
-            horiz_matches[curr_row].append(
-                max_len - matches[lpc].length(), ' ');
+                al.append(padding, ' ');
+            } else {
+                curr_col = 0;
+                add_nl = true;
+            }
         }
-        lnav_data.ld_match_source.replace_with(horiz_matches);
+        lnav_data.ld_match_source.replace_with(al);
     }
 
     tc.reload_data();
