@@ -62,6 +62,7 @@
 #include "ansi_scrubber.hh"
 #include "readline_curses.hh"
 #include "spookyhash/SpookyV2.h"
+#include "fts_fuzzy_match.hh"
 
 using namespace std;
 
@@ -71,6 +72,8 @@ static sig_atomic_t     got_winch   = 0;
 static readline_curses *child_this;
 static sig_atomic_t     looping      = 1;
 static const int        HISTORY_SIZE = 256;
+static int              completion_start;
+static const int        FUZZY_PEER_THRESHOLD = 30;
 
 static const char *RL_INIT[] = {
     /*
@@ -79,6 +82,11 @@ static const char *RL_INIT[] = {
      */
     "set horizontal-scroll-mode on",
     "set bell-style none",
+    "set show-all-if-ambiguous on",
+    "set show-all-if-unmodified on",
+    "set menu-complete-display-prefix on",
+    "TAB: menu-complete",
+    "\"\\e[Z\": menu-complete-backward",
 
     NULL
 };
@@ -168,7 +176,7 @@ static int recvall(int sock, char *buf, size_t len)
     off_t offset = 0;
 
     while (len > 0) {
-        int rc = recv(sock, &buf[offset], len, 0);
+        ssize_t rc = recv(sock, &buf[offset], len, 0);
 
         if (rc == -1) {
             switch (errno) {
@@ -213,28 +221,56 @@ char *readline_context::completion_generator(const char *text, int state)
 {
     static vector<string> matches;
 
-    char *retval = NULL;
+    char *retval = nullptr;
 
     if (state == 0) {
-        int len = strlen(text);
+        auto len = strlen(text);
 
         matches.clear();
-        if (arg_possibilities != NULL) {
-            set<string>::iterator iter;
+        if (arg_possibilities != nullptr) {
+            for (const auto &poss : (*arg_possibilities)) {
+                auto cmpfunc = (loaded_context->is_case_sensitive() ?
+                                strncmp : strncasecmp);
+                auto poss_str = poss.c_str();
 
-            for (iter = arg_possibilities->begin();
-                 iter != arg_possibilities->end();
-                 ++iter) {
-                int (*cmpfunc)(const char *, const char *, size_t);
-                const char *poss_str = iter->c_str();
-
-                cmpfunc = (loaded_context->is_case_sensitive() ?
-                           strncmp : strncasecmp);
                 // Check for an exact match and for the quoted version.
                 if (cmpfunc(text, poss_str, len) == 0 ||
-                    ((strchr(loaded_context->rc_quote_chars, poss_str[0]) != NULL) &&
+                    ((strchr(loaded_context->rc_quote_chars, poss_str[0]) !=
+                        nullptr) &&
                      cmpfunc(text, &poss_str[1], len) == 0)) {
-                    matches.push_back(*iter);
+                    matches.push_back(poss);
+                }
+            }
+
+            if (matches.empty()) {
+                vector<pair<int, string>> fuzzy_matches;
+
+                for (const auto &poss : (*arg_possibilities)) {
+                    string poss_str = tolower(poss);
+                    int score;
+
+                    if (fts::fuzzy_match(text, poss_str.c_str(), score) && score > 0) {
+                        log_debug("match score %d %s %s", score, text, poss.c_str());
+                        if (score <= 0) {
+                            continue;
+                        }
+                        fuzzy_matches.emplace_back(score, poss);
+                    }
+                }
+
+                if (!fuzzy_matches.empty()) {
+                    stable_sort(begin(fuzzy_matches), end(fuzzy_matches),
+                        [](auto l, auto r) { return r.first < l.first; });
+
+                    int highest = fuzzy_matches[0].first;
+
+                    for (const auto &pair : fuzzy_matches) {
+                        if (highest - pair.first < FUZZY_PEER_THRESHOLD) {
+                            matches.push_back(pair.second);
+                        } else {
+                            break;
+                        }
+                    }
                 }
             }
         }
@@ -246,8 +282,8 @@ char *readline_context::completion_generator(const char *text, int state)
 
             last_match_str_valid = false;
             if (sendstring(child_this->rc_command_pipe[readline_curses::RCF_SLAVE],
-                           "m:0:0",
-                           5) == -1) {
+                           "m:0:0:0",
+                           7) == -1) {
                 _exit(1);
             }
         }
@@ -265,8 +301,9 @@ char **readline_context::attempted_completion(const char *text,
                                               int start,
                                               int end)
 {
-    char **retval = NULL;
+    char **retval = nullptr;
 
+    completion_start = start;
     if (start == 0 && loaded_context->rc_possibilities.find("__command") !=
             loaded_context->rc_possibilities.end()) {
         arg_possibilities = &loaded_context->rc_possibilities["__command"];
@@ -322,6 +359,19 @@ static int rubout_char_or_abort(int count, int key)
     }
 }
 
+int readline_context::command_complete(int count, int key)
+{
+    if (loaded_context->rc_possibilities.find("__command") !=
+        loaded_context->rc_possibilities.end()) {
+        char *space = strchr(rl_line_buffer, ' ');
+
+        if (space == nullptr) {
+            return rl_menu_complete(count, key);
+        }
+    }
+    return rl_insert(count, key);
+}
+
 readline_curses::readline_curses()
     : rc_active_context(-1),
       rc_child(-1),
@@ -329,6 +379,73 @@ readline_curses::readline_curses()
       rc_matches_remaining(0),
       rc_max_match_length(0)
 {
+}
+
+readline_curses::~readline_curses()
+{
+    if (this->rc_child == 0) {
+        _exit(0);
+    }
+    else if (this->rc_child > 0) {
+        int status;
+
+        log_debug("term child %d", this->rc_child);
+        kill(this->rc_child, SIGTERM);
+        this->rc_child = -1;
+
+        while (wait(&status) < 0 && (errno == EINTR)) {
+            ;
+        }
+    }
+}
+
+void readline_curses::store_matches(
+    char **matches, int num_matches, int max_len)
+{
+    static int match_index = 0;
+    char msg[64];
+    int rc;
+
+    max_len = 0;
+    for (int lpc = 0; lpc <= num_matches; lpc++) {
+        max_len = max(max_len, (int)strlen(matches[lpc]));
+    }
+
+    if (last_match_str_valid && strcmp(last_match_str.c_str(), matches[0]) == 0) {
+        match_index += 1;
+        rc = snprintf(msg, sizeof(msg), "n:%d", match_index);
+
+        if (sendstring(child_this->rc_command_pipe[RCF_SLAVE], msg, rc) == -1) {
+            _exit(1);
+        }
+    }
+    else {
+        match_index = 0;
+        rc = snprintf(msg, sizeof(msg),
+                      "m:%d:%d:%d",
+                      completion_start, num_matches, max_len);
+        if (sendstring(child_this->rc_command_pipe[RCF_SLAVE], msg, rc) == -1) {
+            _exit(1);
+        }
+        for (int lpc = 1; lpc <= num_matches; lpc++) {
+            if (sendstring(child_this->rc_command_pipe[RCF_SLAVE],
+                           matches[lpc],
+                           strlen(matches[lpc])) == -1) {
+                _exit(1);
+            }
+        }
+
+        last_match_str = matches[0];
+        last_match_str_valid = true;
+    }
+}
+
+void readline_curses::start()
+{
+    if (this->rc_child > 0) {
+        return;
+    }
+
     struct winsize ws;
     int            sp[2];
 
@@ -356,7 +473,13 @@ readline_curses::readline_curses()
         throw error(errno);
     }
 
-    if (this->rc_child == 0) {
+    if (this->rc_child != 0) {
+        this->rc_command_pipe[RCF_SLAVE].reset();
+        this->rc_pty[RCF_SLAVE].reset();
+        return;
+    }
+
+    {
         char buffer[1024];
 
         this->rc_command_pipe[RCF_MASTER].reset();
@@ -377,6 +500,7 @@ readline_curses::readline_curses()
         stifle_history(HISTORY_SIZE);
 
         rl_add_defun("rubout-char-or-abort", rubout_char_or_abort, '\b');
+        // rl_add_defun("command-complete", readline_context::command_complete, ' ');
 
         for (int lpc = 0; RL_INIT[lpc]; lpc++) {
             snprintf(buffer, sizeof(buffer), "%s", RL_INIT[lpc]);
@@ -385,71 +509,8 @@ readline_curses::readline_curses()
 
         child_this = this;
     }
-    else {
-        this->rc_command_pipe[RCF_SLAVE].reset();
-        this->rc_pty[RCF_SLAVE].reset();
-    }
-}
-
-readline_curses::~readline_curses()
-{
-    if (this->rc_child == 0) {
-        _exit(0);
-    }
-    else if (this->rc_child > 0) {
-        int status;
-
-        kill(this->rc_child, SIGTERM);
-        this->rc_child = -1;
-
-        while (wait(&status) < 0 && (errno == EINTR)) {
-            ;
-        }
-    }
-}
-
-void readline_curses::store_matches(
-    char **matches, int num_matches, int max_len)
-{
-    char msg[64];
-    int rc;
-
-    max_len = 0;
-    for (int lpc = 0; lpc <= num_matches; lpc++) {
-        max_len = max(max_len, (int)strlen(matches[lpc]));
-    }
-
-    if (last_match_str_valid && strcmp(last_match_str.c_str(), matches[0]) == 0) {
-        if (sendstring(child_this->rc_command_pipe[RCF_SLAVE], "n", 1) == -1) {
-            _exit(1);
-        }
-    }
-    else {
-        rc = snprintf(msg, sizeof(msg), "m:%d:%d", num_matches, max_len);
-        if (sendstring(child_this->rc_command_pipe[RCF_SLAVE], msg, rc) == -1) {
-            _exit(1);
-        }
-        for (int lpc = 1; lpc <= num_matches; lpc++) {
-            if (sendstring(child_this->rc_command_pipe[RCF_SLAVE],
-                           matches[lpc],
-                           strlen(matches[lpc])) == -1) {
-                _exit(1);
-            }
-        }
-
-        last_match_str = matches[0];
-        last_match_str_valid = true;
-    }
-}
-
-void readline_curses::start(void)
-{
-    if (this->rc_child != 0) {
-        return;
-    }
 
     map<int, readline_context *>::iterator current_context;
-    fd_set rfds;
     int    maxfd;
 
     require(!this->rc_contexts.empty());
@@ -458,17 +519,19 @@ void readline_curses::start(void)
 
     current_context = this->rc_contexts.end();
 
-    FD_ZERO(&rfds);
-    FD_SET(STDIN_FILENO, &rfds);
-    FD_SET(this->rc_command_pipe[RCF_SLAVE], &rfds);
-
     maxfd = max(STDIN_FILENO, this->rc_command_pipe[RCF_SLAVE].get());
 
     while (looping) {
-        fd_set ready_rfds = rfds;
+        fd_set ready_rfds;
         int    rc;
 
-        rc = select(maxfd + 1, &ready_rfds, NULL, NULL, NULL);
+        FD_ZERO(&ready_rfds);
+        if (current_context != this->rc_contexts.end()) {
+            FD_SET(STDIN_FILENO, &ready_rfds);
+        }
+        FD_SET(this->rc_command_pipe[RCF_SLAVE], &ready_rfds);
+
+        rc = select(maxfd + 1, &ready_rfds, nullptr, nullptr, nullptr);
         if (rc < 0) {
             switch (errno) {
             case EINTR:
@@ -481,11 +544,6 @@ void readline_curses::start(void)
 
                 struct itimerval itv;
 
-                if (current_context == this->rc_contexts.end()) {
-                    looping = false;
-                    break;
-                }
-
                 itv.it_value.tv_sec     = 0;
                 itv.it_value.tv_usec    = KEY_TIMEOUT;
                 itv.it_interval.tv_sec  = 0;
@@ -503,10 +561,22 @@ void readline_curses::start(void)
 
                     SpookyHash::Hash128(rl_line_buffer, rl_end, &h1, &h2);
 
+                    if (rl_last_func == readline_context::command_complete) {
+                        rl_last_func = rl_menu_complete;
+                    }
+
+                    bool complete_done = (
+                        rl_last_func != rl_menu_complete &&
+                        rl_last_func != rl_backward_menu_complete);
+
+                    if (complete_done) {
+                        last_match_str_valid = false;
+                    }
+
                     if (h1 == last_h1 && h2 == last_h2) {
                         // do nothing
                     } else if (sendcmd(this->rc_command_pipe[RCF_SLAVE],
-                                       'l',
+                                       complete_done ? 'l': 'c',
                                        rl_line_buffer,
                                        rl_end) != 0) {
                         perror("line: write failed");
@@ -522,16 +592,25 @@ void readline_curses::start(void)
                 if ((rc = recvstring(this->rc_command_pipe[RCF_SLAVE],
                                      msg,
                                      sizeof(msg) - 1)) < 0) {
+                    looping = false;
                 }
                 else {
                     int  context, prompt_start = 0;
                     char type[32];
 
                     msg[rc] = '\0';
-                    if (sscanf(msg, "f:%d:%n", &context, &prompt_start) == 1 &&
-                        prompt_start != 0 &&
-                        (current_context = this->rc_contexts.find(context)) !=
-                        this->rc_contexts.end()) {
+                    if (sscanf(msg, "i:%d:%n", &rl_point, &prompt_start) == 1) {
+                        const char *initial = &msg[prompt_start];
+
+                        rl_extend_line_buffer(strlen(initial) + 1);
+                        strcpy(rl_line_buffer, initial);
+                        rl_end = strlen(initial);
+                        rl_redisplay();
+                    }
+                    else if (sscanf(msg, "f:%d:%n", &context, &prompt_start) == 1 &&
+                             prompt_start != 0 &&
+                             (current_context = this->rc_contexts.find(context)) !=
+                             this->rc_contexts.end()) {
                         current_context->second->load();
                         rl_callback_handler_install(&msg[prompt_start],
                                                     line_ready_tramp);
@@ -569,8 +648,8 @@ void readline_curses::start(void)
                         require(this->rc_contexts[context] != NULL);
 
                         this->rc_contexts[context]->
-                        add_possibility(string(type),
-                                        string(&msg[prompt_start]));
+                                                      add_possibility(string(type),
+                                                                      string(&msg[prompt_start]));
                     }
                     else if (sscanf(msg,
                                     "rp:%d:%31[^:]:%n",
@@ -580,8 +659,8 @@ void readline_curses::start(void)
                         require(this->rc_contexts[context] != NULL);
 
                         this->rc_contexts[context]->
-                        rem_possibility(string(type),
-                                        string(&msg[prompt_start]));
+                                                      rem_possibility(string(type),
+                                                                      string(&msg[prompt_start]));
                     }
                     else if (sscanf(msg, "cp:%d:%s", &context, type)) {
                         this->rc_contexts[context]->clear_possibilities(type);
@@ -655,6 +734,18 @@ void readline_curses::line_ready(const char *line)
     char           msg[1024] = {0};
     int            rc;
 
+    if (line == nullptr) {
+        snprintf(msg, sizeof(msg), "a");
+
+        if (sendstring(this->rc_command_pipe[RCF_SLAVE],
+                       msg,
+                       strlen(msg)) == -1) {
+            perror("abort: write failed");
+            _exit(1);
+        }
+        return;
+    }
+
     if (rl_line_buffer[0] == '^') {
         rc = -1;
     }
@@ -693,7 +784,7 @@ void readline_curses::line_ready(const char *line)
     {
         HIST_ENTRY *entry;
 
-        if (line[0] != '\0' && (
+        if (line != nullptr && line[0] != '\0' && (
             history_length == 0 ||
             (entry = history_get(history_base + history_length - 1)) == NULL ||
             strcmp(entry->line, line) != 0)) {
@@ -728,21 +819,30 @@ void readline_curses::check_poll_set(const vector<struct pollfd> &pollfds)
 
             msg[rc] = '\0';
             if (this->rc_matches_remaining > 0) {
-                this->rc_matches.push_back(msg);
+                this->rc_matches.emplace_back(msg);
                 this->rc_matches_remaining -= 1;
                 if (this->rc_matches_remaining == 0) {
                     this->rc_display_match.invoke(this);
                 }
             }
             else if (msg[0] == 'm') {
-                if (sscanf(msg, "m:%d:%d", &this->rc_matches_remaining,
-                           &this->rc_max_match_length) != 2) {
+                if (sscanf(msg, "m:%d:%d:%d",
+                           &this->rc_match_start,
+                           &this->rc_matches_remaining,
+                           &this->rc_max_match_length) != 3) {
                     require(0);
                 }
                 this->rc_matches.clear();
                 if (this->rc_matches_remaining == 0) {
                     this->rc_display_match.invoke(this);
                 }
+                this->rc_match_index = 0;
+            }
+            else if (msg[0] == 'n') {
+                if (sscanf(msg, "n:%d", &this->rc_match_index) != 1) {
+                    require(0);
+                }
+                this->rc_display_next.invoke(this);
             }
             else {
                 switch (msg[0]) {
@@ -753,8 +853,8 @@ void readline_curses::check_poll_set(const vector<struct pollfd> &pollfds)
                 }
                 switch (msg[0]) {
                 case 'a':
+                    this->vc_line.clear();
                     this->rc_active_context = -1;
-                    this->vc_past_lines.clear();
                     this->rc_matches.clear();
                     this->rc_abort.invoke(this);
                     this->rc_display_match.invoke(this);
@@ -768,7 +868,6 @@ void readline_curses::check_poll_set(const vector<struct pollfd> &pollfds)
 
                 case 'd':
                     this->rc_active_context = -1;
-                    this->vc_past_lines.clear();
                     this->rc_matches.clear();
                     this->rc_perform.invoke(this);
                     this->rc_display_match.invoke(this);
@@ -783,8 +882,10 @@ void readline_curses::check_poll_set(const vector<struct pollfd> &pollfds)
                     this->rc_display_match.invoke(this);
                     break;
 
-                case 'n':
-                    this->rc_display_next.invoke(this);
+                case 'c':
+                    this->rc_line_buffer = &msg[2];
+                    this->rc_change.invoke(this);
+                    this->rc_display_match.invoke(this);
                     break;
                 }
             }
@@ -801,12 +902,9 @@ void readline_curses::handle_key(int ch)
     if (write(this->rc_pty[RCF_MASTER], bch, len) == -1) {
         perror("handle_key: write failed");
     }
-    if (ch == '\t' || ch == '\r') {
-        this->vc_past_lines.clear();
-    }
 }
 
-void readline_curses::focus(int context, const char *prompt)
+void readline_curses::focus(int context, const char *prompt, const char *initial)
 {
     char buffer[1024];
 
@@ -820,14 +918,35 @@ void readline_curses::focus(int context, const char *prompt)
                    strlen(buffer) + 1) == -1) {
         perror("focus: write failed");
     }
-    wmove(this->vc_window, this->get_actual_y(), 0);
+    wmove(this->vc_window, this->get_actual_y(), this->vc_left);
     wclrtoeol(this->vc_window);
+    if (initial != nullptr) {
+        snprintf(buffer, sizeof(buffer), "i:0:%s", initial);
+        if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                       buffer,
+                       strlen(buffer) + 1) == -1) {
+            perror("focus: write failed");
+        }
+    }
+}
+
+void readline_curses::rewrite_line(int pos, std::string value)
+{
+    char buffer[1024];
+
+    snprintf(buffer, sizeof(buffer), "i:%d:%s", pos, value.c_str());
+    if (sendstring(this->rc_command_pipe[RCF_MASTER],
+                   buffer,
+                   strlen(buffer) + 1) == -1) {
+        perror("focus: write failed");
+    }
 }
 
 void readline_curses::abort()
 {
     char buffer[1024];
 
+    this->vc_x = 0;
     snprintf(buffer, sizeof(buffer), "a");
     if (sendstring(this->rc_command_pipe[RCF_MASTER],
                    buffer,
@@ -886,17 +1005,21 @@ void readline_curses::clear_possibilities(int context, string type)
     }
 }
 
-void readline_curses::do_update(void)
+void readline_curses::do_update()
 {
+    if (!this->vc_visible) {
+        return;
+    }
+
     if (this->rc_active_context == -1) {
         int alt_start        = -1;
         struct line_range lr(0, 0);
         attr_line_t       al, alt_al;
 
-        wmove(this->vc_window, this->get_actual_y(), 0);
+        wmove(this->vc_window, this->get_actual_y(), this->vc_left);
         wclrtoeol(this->vc_window);
 
-        if (time(NULL) > this->rc_value_expiration) {
+        if (time(nullptr) > this->rc_value_expiration) {
             this->rc_value.clear();
         }
 
@@ -922,34 +1045,39 @@ void readline_curses::do_update(void)
         lr.lr_end = al.get_string().length();
         view_curses::mvwattrline(this->vc_window,
                                  this->get_actual_y(),
-                                 0,
+                                 this->vc_left,
                                  al,
                                  lr);
         this->set_x(0);
     }
-    vt52_curses::do_update();
 
     if (this->rc_active_context != -1) {
         readline_context *rc = this->rc_contexts[this->rc_active_context];
         readline_highlighter_t hl = rc->get_highlighter();
+        attr_line_t al = this->vc_line;
 
-        if (hl != NULL) {
-            char line[1024];
-            attr_line_t al;
-            string &str = al.get_string();
-            int rc;
-
-            rc = mvwinnstr(this->vc_window,
-                           this->get_actual_y(), 0,
-                           line, sizeof(line));
-
-            str = string(line, rc);
-
-            hl(al, this->vc_x);
-            view_curses::mvwattrline(this->vc_window,
-                this->get_actual_y(), 0, al, line_range(0, rc));
-
-            wmove(this->vc_window, this->get_actual_y(), this->vc_x);
+        if (hl != nullptr) {
+            hl(al, this->vc_left + this->vc_x);
         }
+        view_curses::mvwattrline(this->vc_window,
+                                 this->get_actual_y(), this->vc_left,
+                                 al,
+                                 line_range{ 0, (int) this->vc_width });
+
+        wmove(this->vc_window, this->get_actual_y(), this->vc_left + this->vc_x);
     }
 }
+
+std::string readline_curses::get_match_string() const
+{
+    auto len = this->vc_x - this->rc_match_start;
+    auto context = this->get_active_context();
+
+    if (context->get_append_character() != 0 &&
+        this->rc_line_buffer[this->rc_match_start + len - 2] ==
+        context->get_append_character()) {
+        len -= 2;
+    }
+
+    return this->rc_line_buffer.substr(this->rc_match_start, len);
+}
