@@ -59,12 +59,12 @@ int register_sqlite_funcs(sqlite3 *db, sqlite_registration_func_t *reg_funcs)
 {
     int lpc;
 
-    assert(db != NULL);
-    assert(reg_funcs != NULL);
+    assert(db != nullptr);
+    assert(reg_funcs != nullptr);
     
     for (lpc = 0; reg_funcs[lpc]; lpc++) {
-        struct FuncDef *basic_funcs = NULL;
-        struct FuncDefAgg *agg_funcs = NULL;
+        struct FuncDef *basic_funcs = nullptr;
+        struct FuncDefAgg *agg_funcs = nullptr;
         int i;
 
         reg_funcs[lpc](&basic_funcs, &agg_funcs);
@@ -92,6 +92,8 @@ int register_sqlite_funcs(sqlite3 *db, sqlite_registration_func_t *reg_funcs)
         }
 
         for (i = 0; agg_funcs && agg_funcs[i].zName; i++) {
+            struct FuncDefAgg &fda = agg_funcs[i];
+
             //sqlite3CreateFunc
             sqlite3_create_function(db,
                                     agg_funcs[i].zName,
@@ -101,6 +103,13 @@ int register_sqlite_funcs(sqlite3 *db, sqlite_registration_func_t *reg_funcs)
                                     0,
                                     agg_funcs[i].xStep,
                                     agg_funcs[i].xFinalize);
+
+            if (fda.fda_help.ht_context != HC_NONE) {
+                help_text &ht = fda.fda_help;
+
+                sqlite_function_help.insert(make_pair(ht.ht_name, &ht));
+                ht.index_tags();
+            }
         }
     }
 
@@ -583,6 +592,22 @@ int register_sqlite_funcs(sqlite3 *db, sqlite_registration_func_t *reg_funcs)
                                 .optional())
             .with_parameter(help_text("trigger-name")),
 
+        help_text("INSERT", "Insert rows into a table")
+            .sql_keyword()
+            .with_parameter(help_text("")
+                                .with_flag_name("INTO"))
+            .with_parameter(help_text("schema-name.")
+                                .optional())
+            .with_parameter(help_text("table-name"))
+            .with_parameter(help_text("column-name")
+                                .with_grouping("(", ")")
+                                .zero_or_more())
+            .with_parameter(help_text("expr")
+                                .with_flag_name("VALUES")
+                                .with_grouping("(", ")")
+                                .one_or_more())
+            .with_example({"INSERT INTO environ VALUES ('MSG', 'HELLO, WORLD!')"}),
+
         help_text("SELECT",
                   "Query the database and return zero or more rows of data.")
             .sql_keyword()
@@ -627,7 +652,8 @@ int register_sqlite_funcs(sqlite3 *db, sqlite_registration_func_t *reg_funcs)
                                 .one_or_more())
             .with_parameter(help_text("cond", "The condition used to determine whether a row should be updated.")
                                 .with_flag_name("WHERE")
-                                .optional()),
+                                .optional())
+            .with_example({"UPDATE syslog_log SET log_mark = 1 WHERE log_line = 40"}),
 
         help_text("CASE",
                   "Evaluate a series of expressions in order until one evaluates to true and then return it's result.  "
