@@ -76,7 +76,7 @@ static string timeslice(const char *time_in, nonstd::optional<const char *> slic
         throw sqlite_func_error("unable to parse time value");
     }
 
-    int64_t us = tv.tv_sec * 1000 * 1000 + tv.tv_usec, remainder;
+    int64_t us = tv.tv_sec * 1000000LL + tv.tv_usec, remainder;
 
     remainder = us % rt.to_microseconds();
     us -= remainder;
@@ -99,15 +99,13 @@ nonstd::optional<double> sql_timediff(const char *time1, const char *time2)
     struct relative_time::parse_error pe;
 
     if (rt1.parse(time1, -1, pe)) {
-        rt1.add_now()
-            .to_timeval(tv1);
+        tv1 = rt1.add_now().to_timeval();
     } else if (!dts1.convert_to_timeval(time1, -1, NULL, tv1)) {
         return nonstd::nullopt;
     }
 
     if (rt2.parse(time2, -1, pe)) {
-        rt2.add_now()
-            .to_timeval(tv2);
+        tv2 = rt2.add_now().to_timeval();
     } else if (!dts2.convert_to_timeval(time2, -1, NULL, tv2)) {
         return nonstd::nullopt;
     }
