@@ -31,6 +31,7 @@
 
 #include "config.h"
 
+#include <cmath>
 #include <string>
 
 #include "auto_mem.hh"
@@ -172,9 +173,10 @@ bool rgb_color::from_str(const string_fragment &color,
 
 string_attr_type view_curses::VC_STYLE("style");
 string_attr_type view_curses::VC_GRAPHIC("graphic");
+string_attr_type view_curses::VC_FOREGROUND("foreground");
+string_attr_type view_curses::VC_BACKGROUND("background");
 
-const struct itimerval ui_periodic_timer::INTERVAL = {
-    { 0, 350 * 1000 },
+const struct itimerval ui_periodic_timer::INTERVAL = {    { 0, 350 * 1000 },
     { 0, 350 * 1000 }
 };
 
@@ -411,6 +413,11 @@ void view_curses::mvwattrline(WINDOW *window,
     tab_count     = count(line.begin(), line.end(), '\t');
     expanded_line = (char *)alloca(line.size() + tab_count * 8 + 1);
 
+    unsigned char *fg_color = (unsigned char *) alloca(line_width);
+    bool has_fg = false;
+    unsigned char *bg_color = (unsigned char *) alloca(line_width);
+    bool has_bg = false;
+
     for (size_t lpc = 0; lpc < line.size(); lpc++) {
         int exp_start_index = exp_index;
         unsigned char ch = static_cast<unsigned char>(line[lpc]);
@@ -481,7 +488,10 @@ void view_curses::mvwattrline(WINDOW *window,
         require(attr_range.lr_start >= 0);
         require(attr_range.lr_end >= -1);
 
-        if (!(iter->sa_type == &VC_STYLE || iter->sa_type == &VC_GRAPHIC)) {
+        if (!(iter->sa_type == &VC_STYLE ||
+              iter->sa_type == &VC_GRAPHIC ||
+              iter->sa_type == &VC_FOREGROUND ||
+              iter->sa_type == &VC_BACKGROUND)) {
             continue;
         }
 
@@ -515,8 +525,25 @@ void view_curses::mvwattrline(WINDOW *window,
             continue;
         }
 
+        if (iter->sa_type == &VC_FOREGROUND) {
+            if (!has_fg) {
+                memset(fg_color, COLOR_WHITE, line_width);
+            }
+            fill(fg_color + attr_range.lr_start, fg_color + attr_range.lr_end, iter->sa_value.sav_int);
+            has_fg = true;
+            continue;
+        }
+
+        if (iter->sa_type == &VC_BACKGROUND) {
+            if (!has_bg) {
+                memset(bg_color, COLOR_BLACK, line_width);
+            }
+            fill(bg_color + attr_range.lr_start, bg_color + attr_range.lr_end, iter->sa_value.sav_int);
+            has_bg = true;
+            continue;
+        }
+
         if (attr_range.lr_end > attr_range.lr_start) {
-            string_attrs_t::const_iterator range_iter;
             int awidth = attr_range.length();
             int color_pair;
 
@@ -530,6 +557,11 @@ void view_curses::mvwattrline(WINDOW *window,
 
                 mvwin_wchnstr(window, y, x_pos, row_ch, ch_width);
                 for (int lpc = 0; lpc < ch_width; lpc++) {
+                    bool clear_rev = false;
+
+                    if (row_ch[lpc].attr & A_REVERSE && attrs & A_REVERSE) {
+                        clear_rev = true;
+                    }
                     if (color_pair > 0) {
                         row_ch[lpc].attr =
                             attrs | (row_ch[lpc].attr & ~A_COLOR);
@@ -541,11 +573,42 @@ void view_curses::mvwattrline(WINDOW *window,
                     } else {
                         row_ch[lpc].attr |= attrs;
                     }
+                    if (clear_rev) {
+                        row_ch[lpc].attr &= ~A_REVERSE;
+                    }
                 }
                 mvwadd_wchnstr(window, y, x_pos, row_ch, ch_width);
             }
         }
     }
+
+#if 1
+    if (has_fg || has_bg) {
+        if (!has_fg) {
+            memset(fg_color, COLOR_WHITE, line_width);
+        }
+        if (!has_bg) {
+            memset(bg_color, COLOR_BLACK, line_width);
+        }
+
+        int x_pos = x + lr.lr_start;
+        int ch_width = lr.length();
+        cchar_t row_ch[ch_width + 1];
+
+        mvwin_wchnstr(window, y, x_pos, row_ch, ch_width);
+        for (int lpc = 0; lpc < ch_width; lpc++) {
+            int color_pair = view_colors::ansi_color_pair_index(fg_color[lpc], bg_color[lpc]);
+
+            row_ch[lpc].attr = row_ch[lpc].attr & ~A_COLOR;
+#ifdef NCURSES_EXT_COLORS
+            row_ch[lpc].ext_color = color_pair;
+#else
+            row_ch[lpc].attr |= COLOR_PAIR(color_pair);
+#endif
+        }
+        mvwadd_wchnstr(window, y, x_pos, row_ch, ch_width);
+    }
+#endif
 }
 
 class color_listener : public lnav_config_listener {
@@ -561,13 +624,13 @@ attr_t view_colors::BASIC_HL_PAIRS[view_colors::BASIC_COLOR_COUNT] = {
     ansi_color_pair(COLOR_CYAN, COLOR_BLACK),
     ansi_color_pair(COLOR_GREEN, COLOR_BLACK),
     ansi_color_pair(COLOR_MAGENTA, COLOR_BLACK),
-    ansi_color_pair(COLOR_BLUE, COLOR_WHITE),
+    ansi_color_pair(COLOR_BLACK, COLOR_WHITE),
     ansi_color_pair(COLOR_CYAN, COLOR_BLACK),
-    ansi_color_pair(COLOR_GREEN, COLOR_WHITE),
-    ansi_color_pair(COLOR_MAGENTA, COLOR_WHITE),
+    ansi_color_pair(COLOR_YELLOW, COLOR_MAGENTA) | A_BOLD,
+    ansi_color_pair(COLOR_MAGENTA, COLOR_CYAN) | A_BOLD,
 };
 
-view_colors &view_colors::singleton(void)
+view_colors &view_colors::singleton()
 {
     static view_colors s_vc;
 
@@ -634,17 +697,21 @@ void view_colors::init(void)
 
 inline attr_t attr_for_colors(int &pair_base, short fg, short bg)
 {
-    int pair = ++pair_base;
-
-    if (lnav_config.lc_ui_default_colors) {
-        if (fg == COLOR_WHITE) {
-            fg = -1;
-        }
-        if (bg == COLOR_BLACK) {
-            bg = -1;
+    if (COLOR_PAIRS <= 64) {
+        return view_colors::ansi_color_pair(fg, bg);
+    } else {
+        if (lnav_config.lc_ui_default_colors) {
+            if (fg == COLOR_WHITE) {
+                fg = -1;
+            }
+            if (bg == COLOR_BLACK) {
+                bg = -1;
+            }
         }
     }
 
+    int pair = ++pair_base;
+
     init_pair(pair, fg, bg);
 
     return COLOR_PAIR(pair);
@@ -652,6 +719,9 @@ inline attr_t attr_for_colors(int &pair_base, short fg, short bg)
 
 void view_colors::init_roles(int color_pair_base)
 {
+    rgb_color fg, bg;
+    string err;
+
     /* Setup the mappings from roles to actual colors. */
     this->vc_role_colors[VCR_TEXT] =
         attr_for_colors(color_pair_base, COLOR_WHITE, COLOR_BLACK);
@@ -683,11 +753,18 @@ void view_colors::init_roles(int color_pair_base)
     this->vc_role_colors[VCR_VIEW_STATUS] =
         attr_for_colors(color_pair_base, COLOR_WHITE, COLOR_BLUE) | A_BOLD;
 
+    rgb_color::from_str(string_fragment("White"), fg, err);
+    rgb_color::from_str(string_fragment("Grey37"), bg, err);
+    this->vc_role_colors[VCR_INACTIVE_STATUS] = ensure_color_pair(fg, bg);
+
+    this->vc_role_colors[VCR_POPUP] =
+        attr_for_colors(color_pair_base, COLOR_WHITE, COLOR_CYAN) | A_BOLD;
+
     this->vc_role_colors[VCR_KEYWORD] = attr_for_colors(color_pair_base, COLOR_BLUE, COLOR_BLACK);
     this->vc_role_colors[VCR_STRING] = attr_for_colors(color_pair_base, COLOR_GREEN, COLOR_BLACK) | A_BOLD;
     this->vc_role_colors[VCR_COMMENT] = attr_for_colors(color_pair_base, COLOR_GREEN, COLOR_BLACK);
     this->vc_role_colors[VCR_VARIABLE] = attr_for_colors(color_pair_base, COLOR_CYAN, COLOR_BLACK);
-    this->vc_role_colors[VCR_SYMBOL] = attr_for_colors(color_pair_base, COLOR_MAGENTA, COLOR_BLACK);
+    this->vc_role_colors[VCR_SYMBOL] = attr_for_colors(color_pair_base, COLOR_MAGENTA, COLOR_BLACK) | A_BOLD;
     this->vc_role_colors[VCR_RE_SPECIAL] = attr_for_colors(color_pair_base, COLOR_CYAN, COLOR_BLACK);
     this->vc_role_colors[VCR_RE_REPEAT] = attr_for_colors(color_pair_base, COLOR_YELLOW, COLOR_BLACK);
     this->vc_role_colors[VCR_FILE] = attr_for_colors(color_pair_base, COLOR_BLUE, COLOR_BLACK);
@@ -710,3 +787,46 @@ int view_colors::ensure_color_pair(const rgb_color &rgb_fg, const rgb_color &rgb
         xterm_colors.match_color(rgb_fg),
         rgb_bg.empty() ? (short) COLOR_BLACK : xterm_colors.match_color(rgb_bg));
 }
+
+lab_color::lab_color(const rgb_color &rgb)
+{
+    double r = rgb.rc_r / 255.0,
+        g = rgb.rc_g / 255.0,
+        b = rgb.rc_b / 255.0,
+        x, y, z;
+
+    r = (r > 0.04045) ? pow((r + 0.055) / 1.055, 2.4) : r / 12.92;
+    g = (g > 0.04045) ? pow((g + 0.055) / 1.055, 2.4) : g / 12.92;
+    b = (b > 0.04045) ? pow((b + 0.055) / 1.055, 2.4) : b / 12.92;
+
+    x = (r * 0.4124 + g * 0.3576 + b * 0.1805) / 0.95047;
+    y = (r * 0.2126 + g * 0.7152 + b * 0.0722) / 1.00000;
+    z = (r * 0.0193 + g * 0.1192 + b * 0.9505) / 1.08883;
+
+    x = (x > 0.008856) ? pow(x, 1.0/3.0) : (7.787 * x) + 16.0/116.0;
+    y = (y > 0.008856) ? pow(y, 1.0/3.0) : (7.787 * y) + 16.0/116.0;
+    z = (z > 0.008856) ? pow(z, 1.0/3.0) : (7.787 * z) + 16.0/116.0;
+
+    this->lc_l = (116.0 * y) - 16;
+    this->lc_a = 500.0 * (x - y);
+    this->lc_b = 200.0 * (y - z);
+}
+
+double lab_color::deltaE(const lab_color &other) const
+{
+    double deltaL = this->lc_l - other.lc_l;
+    double deltaA = this->lc_a - other.lc_a;
+    double deltaB = this->lc_b - other.lc_b;
+    double c1 = sqrt(this->lc_a * this->lc_a + this->lc_b * this->lc_b);
+    double c2 = sqrt(other.lc_a * other.lc_a + other.lc_b * other.lc_b);
+    double deltaC = c1 - c2;
+    double deltaH = deltaA * deltaA + deltaB * deltaB - deltaC * deltaC;
+    deltaH = deltaH < 0.0 ? 0.0 : sqrt(deltaH);
+    double sc = 1.0 + 0.045 * c1;
+    double sh = 1.0 + 0.015 * c1;
+    double deltaLKlsl = deltaL / (1.0);
+    double deltaCkcsc = deltaC / (sc);
+    double deltaHkhsh = deltaH / (sh);
+    double i = deltaLKlsl * deltaLKlsl + deltaCkcsc * deltaCkcsc + deltaHkhsh * deltaHkhsh;
+    return i < 0.0 ? 0.0 : sqrt(i);
+}
