@@ -161,7 +161,9 @@ int main(int argc, char *argv[]) {
               break;
         case MODE_LEVELS:
           for (logfile::iterator iter = lf.begin(); iter != lf.end(); ++iter) {
-            printf("0x%02x\n", iter->get_level_and_flags());
+            log_level_t level = iter->get_level_and_flags();
+            printf("%s 0x%x\n", level_names[level & ~LEVEL__FLAGS],
+                level & LEVEL__FLAGS);
           }
               break;
       }
